UNIT cationdisorder;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Calculating contributions to properties due to cation disorder in the
   multiple Einstein method, based on:
   O'Neil HStC, Navrotsky A (1983) Am Mineral 68:181-194 and
   O'Neil HStC, Navrotsky A (1984) Am Mineral 69:733-753

   CATION_DO[1]: 1st coefficient interchange enthalpy
   CATION_DO[2]: 2nd coefficient interchange enthalpy
   CATION_DO[3]: 1st coefficient interchange entropy
   CATION_DO[4]: 2nd coefficient interchange entropy
   CATION_DO[5]: # moles tetrahedral sites per molecular formula unit
   CATION_DO[6]: # moles octahedral sites per molecular formula unit
 *)

INTERFACE
USES CRT,sysutils,variables;


PROCEDURE DISORDER_CALC(T:EXTENDED;VAR GDIS,HDIS,UDIS,SDIS,CPDIS,CVDIS:EXTENDED);
IMPLEMENTATION

FUNCTION EQUIL_DISORDER(T,Y:EXTENDED):EXTENDED;
(* Equilibrium condition for disorder obtained from (dGdis/dY)P,T=0
   See P3267-3271 of Notes, p371-376 Wallace Notes, eqn (21) Helmholtz.doc.
   T=temperature, Y=site_fraction of metal in tetrahedral sites.
 *)
VAR Z,ZH,ZS,Q,L:EXTENDED;
BEGIN
  ZH:=CATION_DO[1]+2*CATION_DO[2]*Y;
  ZS:=CATION_DO[3]+2*CATION_DO[4]*Y;
  (* L=N_oct/N_tet, eqn. (16.5) *)
  L:=CATION_DO[6]/CATION_DO[5];
  Z:=Y*Y/(1-Y)/(L-Y); Q:=CATION_DO[5]*GAS_CONSTANT*T*LN(Z);
  (* Eqn. (16.15) for determining site fraction y(M,tet) *)
  EQUIL_DISORDER:=ABS(ZH+Q-T*ZS)
END;

PROCEDURE CHECKCOUNT(VAR CO:INTEGER;VAR II:EXTENDED;VAR LONG_IT:BOOLEAN);
(* Expands step value in minimization *)
VAR M,J:INTEGER;
BEGIN
  M:=5;
  IF CO>M THEN
  BEGIN
    IF ABS(II)<1E-2 THEN
    BEGIN
      IF II<0 THEN J:=-1 ELSE J:=1;
      II:=3*II; CO:=0; LONG_IT:=TRUE;
      IF ABS(II)>1E-2 THEN II:=J*1E-2
    END
  END
END;

PROCEDURE TROUBLE_Y(VAR Y:EXTENDED);
(* If the site_fraction Y=0 or 1 then the function equil_disorder is not
   defined. This procedure keeps site fraction between [0,1]
 *)
VAR D:EXTENDED;
BEGIN
  D:=1E-12;
  IF Y>=1-D THEN Y:=1-D;
  IF Y<=D THEN Y:=D
END;

PROCEDURE MINIMIZE(I1,T:EXTENDED;VAR Y:EXTENDED);
(* One dimensional minimization of a target function abs(EQUIL_DISORDER(T,Y)).
   Y is the site fraction of the metal ion in tetrahedral sites
 *)
VAR I,GL,GN,YM,YLAST:EXTENDED;SW,LONG:BOOLEAN;COUNTER:INTEGER;
BEGIN
  I:=I1;LONG:=FALSE;SW:=FALSE;
  REPEAT
    TROUBLE_Y(Y); YM:=Y; COUNTER:=0;
    IF LONG=TRUE THEN I:=I/3;
    IF (LONG=TRUE) AND (ABS(I)<=I1) THEN LONG:=FALSE;
    IF SW=FALSE THEN GL:=EQUIL_DISORDER(T,Y);
    Y:=Y+I; TROUBLE_Y(Y); SW:=TRUE;
    GN:=EQUIL_DISORDER(T,Y);
    YLAST:=Y;
    IF GN>=GL THEN
    BEGIN
      I:=-I; Y:=YM; GN:=GL
    END;
    REPEAT
      GL:=GN; YLAST:=Y; Y:=Y+I; TROUBLE_Y(Y); COUNTER:=COUNTER+1;
      GN:=EQUIL_DISORDER(T,Y);
      CHECKCOUNT(COUNTER,I,LONG)
    UNTIL
      (GN>=GL);
    Y:=YLAST
  UNTIL
    (LONG=FALSE);
  I:=I1
END;

PROCEDURE GET_MINIMUM(T:EXTENDED;VAR Y:EXTENDED);
(* Controls the steps in the minimization *)
VAR I1:EXTENDED;
BEGIN
  Y:=0.1; I1:=0.003;
  WHILE I1>1E-11 DO
  BEGIN
    MINIMIZE(I1,T,Y); I1:=I1/3
  END
END;

PROCEDURE DISORDER_CALC(T:EXTENDED;VAR GDIS,HDIS,UDIS,SDIS,CPDIS,CVDIS:EXTENDED);
(* O'Neill and Navrotsky (1983), Am Mineral 68:181-194.
   Calculation of thermodynamic properties for cation disorder.
   The Gibbs energy of disorder is deltaGdis=G(P,T,Y)-G(P,T,Y=0)=G(P,T,Y)-0.
   The effect of pressure is neglected. Therefore deltaVdis=(d(deltaG)/dP)T=
   (d(deltaG)/dY)T*(dY/dP)T=0 and deltaHdis=deltaUdis-P*deltaVdis=deltaUdis.
 *)
VAR DELTA,Y,Z0,Z1,Z2,Z3,SEDIS,DYDT,L:EXTENDED;
BEGIN
  GDIS:=0; HDIS:=0; UDIS:=0; SDIS:=0; CPDIS:=0; CVDIS:=0; DELTA:=1E-12;
  IF T>DELTA THEN GET_MINIMUM(T,Y) ELSE Y:=0;
  IF Y>DELTA THEN
  BEGIN
    (* L=(#octaeder_sites/#tetraeder_sites), eqn. (16.5). *)
    L:=CATION_DO[6]/CATION_DO[5];
    (* Enthalpy, eqn. (16.7). *)
    HDIS:=(CATION_DO[1]+CATION_DO[2]*Y)*Y;
    (* Non-configurational part of the entropy, eqn. (16.12). *)
    SEDIS:=(CATION_DO[3]+CATION_DO[4]*Y)*Y;
    Z1:=(1-Y)*LN(1-Y)+Y*LN(Y); Z1:=Z1*CATION_DO[5];
    Z2:=(Y/L)*LN(Y/L)+(1-Y/L)*LN(1-Y/L); Z2:=Z2*CATION_DO[6];
    (* Entropy, eqn. (16.12). *)
    SDIS:=-GAS_CONSTANT*(Z1+Z2)+SEDIS;
    (* Gibbs energy, eqn. (16.13). *)
    GDIS:=HDIS-T*SDIS;
    (* Energy, eqn. (16.6). *)
    UDIS:=HDIS;
    (* Heat capacity Cp, eqn. (16.16)+(16.17). *)
    Z0:=CATION_DO[1]+2*CATION_DO[2]*Y;
    Z1:=Y*(1-Y)*(L-Y);
    Z2:=2*L-Y*(1+L);
    Z2:=Z2/Z1;
    Z3:=2*CATION_DO[2]*T-2*CATION_DO[4]*T*T+CATION_DO[5]*GAS_CONSTANT*T*T*Z2;
    DYDT:=Z0/Z3;
    CPDIS:=Z0*DYDT;
    (* Heat capacity Cv=Cp *)
    CVDIS:=CPDIS
  END
END;

END.
