UNIT clonevdos;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Reducing the number of Einstein frequencies in the VDoS: cloning the original
   description.

1. MAKE_NEW_DATABASE: main control
2. TARGET_PROPERTIES
   Set entropy and entropy at the target condition P_target, T_target
3. ASSIGN_PARAMETERS:
   Copy XiPT frequencies and fractions to wavenumber[i], frac[i]
4. MAKE_EXTRAVDOS_POINTS
   The N_EINSTEIN modes in original VDoS are extended with (0,0), 2nd point having
   zero fraction, and last point just after the largest frequency having zero
   fraction => (X_exp,G_exp) 1..max_points
5. MAKE_SETUP
   Define DELTA as the half-width of each Einstein box for the clone.
   In:
5a. MAKE_EINSTEIN_AVERAGE:
    Set 19 points between 2 points in (X_exp,G_exp) to form (X_LARGE,G_LARGE)
    1..max_points_large.
    With the defined DELTA determine fractions of the begin and end frequency of
    the box. The fraction of the box is determined by averaging all fractions
    between these 2 end points. The VDoS of the clone is then defined by
    (X_cal,G_cal) 1..NUMEIN_CLONE
5. MAKE_SETUP again
   Store X_cal in theta_wave[i]
6. ASSIGN_BACK
   Assign frequencies in theta_wave and fractions in G_cal to 'me'  variables
   THETA0 and FRAC
7. DISPERSION_AVERAGE
   Determine Gruneisen, mode-q, anharmonicity when dispersion by simple averaging
8. STRETCH_VDOS
   Stretch the VDoS of the clone until the target entropy is achieved.
 *)
INTERFACE
USES crt,sysutils,variables,makevolume,savemef;

PROCEDURE CONVERT_VDOS;
IMPLEMENTATION

TYPE RY_WAVE=ARRAY[1..1000] OF EXTENDED;
     RY_LARGE=ARRAY[1..12000] OF EXTENDED;
VAR DELTA,OLD_STATIC_VOLUME,H_TARGET,S_TARGET,V_TARGET,H_TARGET_CAL,S_TARGET_CAL,
    V_TARGET_CAL:EXTENDED;
                             (* DELTA     : Half-width of an optic continuum
                                OLD_STATIC_VOLUME: Static volume for endmember/compound for
                                 for original database
                                H_TARGET,S_TARGET,V_TARGET: enthalpy, entropy volume at the
                                 target conditions T_clone, P_clone calculated with original
                                 VDoS
                                H_TARGET_CAL,S_TARGET_CAL,V_TARGET_CAL: calculated
                                 enthalpy, entropy volume at T_clone, P_clone with
                                 cloned VDoS
                              *)
    THETA_WAVE,FRAC_WAVE,WAVE_CONVERT,X_EXP,G_EXP,X_CAL,G_CAL,OLD_THETA_WAVE,
    X_GRUN,G_GRUN:RY_WAVE;
                             (* THETA_WAVE: Einstein temperature of input VDoS
                                FRAC_WAVE: fractions of input VDoS
                                WAVE_CONVERT: Intermediate array for wave_numbers
                                X_EXP: Wavenumber of input VDoS (middle continuum or Einstein)
                                G_EXP: VDoS value of input VDoS
                                X_CAL: Frequency of the middle of an OC mode
                                G_CAL: its height, selected
                                OLD_WAVE_NUMBER: Wavenumbers determined in make_vdos_points
                                 as middle of continua or Einsteins: these appear in the
                                 original VDoS and their size is the number of modes
                                X_GRUN,G_GRUN is wavenumber, Gruneisen parameter in original
                                 VDoS
                              *)
    NUM_FREQ,MAX_POINTS,OLD_NUM_FREQ,MAX_POINTS_LARGE:INTEGER;
                              (* NUM_FREQ: number of Einstein frequencies
                                 MAX_POINTS: number of points extracted from input VDOS
                                 OLD_NUM_FREQ: number of Einstein modes in the original VDoS
                                 MAX_POINTS_LARGE: Original VDoS contains MAX_POINTS points
                                  and is extended with extra points to MAX_POINTS_LARGE
                               *)
    PHASE_DISPERSE:BOOLEAN;
                              (* PHASE_DISPERSE: True when dispersion in Gruneisen
                                  parameters
                               *)
    X_LARGE,G_LARGE:RY_LARGE;
                              (* VDoS of original database extended with 9 extra points
                                  linearly interpolated between 2 points
                               *)


PROCEDURE INITIALIZE_DISPERSION;
(* Initializes array showing that dispersion in Gruneisen parameters is present *)
BEGIN
  PHASE_DISPERSE:=FALSE
END;

PROCEDURE TARGET_PROPERTIES;
(* Calculates target properties, entropy and enthalpy at target conditions, e.g.
   at 298.15 K, and 1 bar. They must remain the same in the new VDoS description
 *)
VAR VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,DPDT,ALPHA,
    ZERO_PNT:EXTENDED;
BEGIN
  (* Set target conditions, Enthalpy and Entropy must be kept identical at
     the target condition e.g. 298.15 K and 1 bar
   *)
  TEMPERATURE:=T_CLONE; PRESSURE:=P_CLONE;
  VPT:=V0;
  GIBBS:=0; HELMHOLTZ:=0; ENTHALPY:=0; ENERGY:=0; ENTROPY:=0; CP:=0; CV:=0;
  BULKMOD:=0; DPDT:=0; ALPHA:=0; ZERO_PNT:=0;
  GET_THERMO_PROPERTIES(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,
                        BULKMOD,DPDT,ALPHA,ZERO_PNT);
  S_TARGET:=ENTROPY; H_TARGET:=ENTHALPY; V_TARGET:=VPT; OLD_STATIC_VOLUME:=V0_ST
END;

PROCEDURE ASSIGN_PARAMETERS;
(* Assign 'me' parameters to new Einstein temperatures and fraction variables 
 *)
VAR J:INTEGER;
BEGIN
  (* The original VDoS has N_EINSTEIN frequencies *)
  NUM_FREQ:=N_EINSTEIN; OLD_NUM_FREQ:=NUM_FREQ;
  FOR J:=1 TO NUM_FREQ DO THETA_WAVE[J]:=THETA0[J];
  (* Fractions *)
  FOR J:=1 TO NUM_FREQ DO FRAC_WAVE[J]:=FRAC[J]
END;

PROCEDURE MAKE_EXTRAVDOS_POINTS;
(* Store the original VDoS of the original description as (x_exp[j], g_exp[j]),
   There are NUM_FREQ monochromators with Einstein temperatures THETA_WAVE[1..num_freq]
   and fractions FRAC_WAVE[1..num_freq].
 *)
VAR FRAC_CONVERT:RY_WAVE;
    X:EXTENDED;
    I,J,NUM:INTEGER;
    STOP_IT:BOOLEAN;
BEGIN
  (* Copy array THETA_WAVE to array OLD_THETA_WAVE to be used in dispersion_average *)
  OLD_THETA_WAVE:=THETA_WAVE;
  (* We need as first point (0,0), as second point one that has zero fraction extrapolated
     from the VDoS and as last point zero fraction extrapolated from VDoS
   *)
  NUM:=1; WAVE_CONVERT[1]:=0; FRAC_CONVERT[1]:=0;
  DELTA:=(THETA_WAVE[2]-THETA_WAVE[1])/2;
  (* 2nd point if necessary *)
  X:=THETA_WAVE[1]-DELTA;
  IF X>1E-6 THEN
  BEGIN
    NUM:=2; WAVE_CONVERT[2]:=X; FRAC_CONVERT[2]:=0
  END;
  (* Store rest of VDoS in help array *)
  FOR J:=1 TO NUM_FREQ DO
  BEGIN
    WAVE_CONVERT[J+NUM]:=THETA_WAVE[J];
    FRAC_CONVERT[J+NUM]:=FRAC_WAVE[J]
  END;
  NUM_FREQ:=NUM_FREQ+NUM;
  (* Last point extrapolated from VDoS, search fraction unequal zero from the last mode.
     That makes VDoS somewhat smaller so that we have slightly thinner continua
   *)
  J:=NUM_FREQ+1; STOP_IT:=FALSE;
  REPEAT
    J:=J-1;
    IF FRAC_CONVERT[J]<1E-12 THEN
    BEGIN
      NUM_FREQ:=NUM_FREQ-1
    END
    ELSE STOP_IT:=TRUE;
  UNTIL
    (STOP_IT=TRUE) OR (J<2);
  DELTA:=(WAVE_CONVERT[NUM_FREQ]-WAVE_CONVERT[NUM_FREQ-1])/2;
  X:=WAVE_CONVERT[NUM_FREQ]+DELTA;
  NUM_FREQ:=NUM_FREQ+1; WAVE_CONVERT[NUM_FREQ]:=X; FRAC_CONVERT[NUM_FREQ]:=0;
  (* Copy wave_convert and frac_convert back to wave_number, frac *)
  FOR J:=1 TO NUM_FREQ DO
  BEGIN
    THETA_WAVE[J]:=WAVE_CONVERT[J]; FRAC_WAVE[J]:=FRAC_CONVERT[J]
  END;
  (* Make VDoS points required for VDoS.Pas *)
  FOR I:=1 TO NUM_FREQ DO
  BEGIN
    X_EXP[I]:=THETA_WAVE[I]; G_EXP[I]:=FRAC_WAVE[I]
  END;
  MAX_POINTS:=NUM_FREQ
END;

PROCEDURE MAKE_LARGE_EXP;
(* The VDoS of the original database is extended with 19 extra points between two
   subsequent points in a linear way. Purpose is to make averaging more accurate.
   We have MAX_POINTS in the VDoS given as (x_exp[i],g_exp[i]) and change the VDoS
   to (X_LARGE[I],G_LARGE[I]) with MAX_POINTS_LARGE points
 *)
VAR I,J,N:INTEGER;
    A,D,Y:EXTENDED;
BEGIN
  N:=0;
  FOR I:=1 TO MAX_POINTS-1 DO
  BEGIN
    D:=(X_EXP[I+1]-X_EXP[I])/20;
    N:=N+1; X_LARGE[N]:=X_EXP[I]; G_LARGE[N]:=G_EXP[I];
    (* Store 19 points subsequently, so not x_exp[i+1]: that comes in the next cycle *)
    FOR J:=1 TO 19 DO
    BEGIN
      Y:=X_EXP[I]+J*D; N:=N+1;
      X_LARGE[N]:=Y;
      A:=(G_EXP[I+1]-G_EXP[I])/(X_EXP[I+1]-X_EXP[I]);
      G_LARGE[N]:=A*(Y-X_EXP[I])+G_EXP[I]
    END;
  END;
  (* Store last point *)
  N:=N+1; X_LARGE[N]:=X_EXP[MAX_POINTS]; G_LARGE[N]:=G_EXP[MAX_POINTS];
  MAX_POINTS_LARGE:=N
END;

PROCEDURE GET_NEAREST_LARGE(X:EXTENDED;VAR J_LOW,J_UP:INTEGER);
(* X is the mid-point of an Einstein box. J_low is the number of the point in the
   extended VDoS (original VDoS extended with extra points linearly interpolated
   between 2 points, X_large,G_large), just below X and J_UP the number of the point
   just higher as X
 *)
VAR J:INTEGER;STOP_IT:BOOLEAN;
BEGIN
  J:=0; STOP_IT:=FALSE;
  REPEAT
    J:=J+1;
    IF X_LARGE[J]>X THEN STOP_IT:=TRUE
  UNTIL
    (J>=MAX_POINTS_LARGE) OR (STOP_IT=TRUE);
  J_UP:=J; J_LOW:=J-1;
  IF J_LOW=0 THEN
  BEGIN
    (* In unforseen case when the experimental points start at values above X *)
    J_LOW:=1; J_UP:=2
  END
END;

PROCEDURE GET_BETWEEN_SUM(X0:EXTENDED;VAR N_BETWEEN:INTEGER;VAR SUM_BETWEEN:EXTENDED);
(* Gets the number of points between <X0-DELTA,X0+DELTA>. Gets the sum of the VDoS
   values (extended VDoS for averaging method) in that interval: sum_between
 *)
VAR X_LEFT,X_RIGHT,D1:EXTENDED;
    J:INTEGER;
BEGIN
  N_BETWEEN:=0; SUM_BETWEEN:=0; D1:=1E-14;
  X_LEFT:=X0-DELTA; X_RIGHT:=X0+DELTA;
  IF X_LEFT<0 THEN X_LEFT:=0;
  FOR J:=1 TO MAX_POINTS_LARGE DO
  BEGIN
    IF (X_LARGE[J]>X_LEFT+D1) AND (X_LARGE[J]<X_RIGHT-D1) THEN
    BEGIN
      N_BETWEEN:=N_BETWEEN+1;
      SUM_BETWEEN:=SUM_BETWEEN+G_LARGE[J]
    END
  END
END;

PROCEDURE MAKE_EINSTEIN_AVERAGE;
(* Make arrays containing the midpoint frequency of Einstein boxes and their average
   value in the block [x_cal[i]-delta,x_cal[i]+delta]. The width of an Einstein box
   is the distance between 2 subsequent frequencies. Delta is determined in MAKE_SETUP.
   The height of a box is the average of the heights in the original VDoS in the 
   frequency range of the clone box: that will be the fraction.
   This is the averaging method in XiPT. The endresult is x_cal[i],frac[i]
   and i=[1..NUMEIN_CLONE]
 *)
VAR G,X0,SUM,X_BOUND,G_LEFT,G_RIGHT,SUM_BETWEEN:EXTENDED;
    I,J_LOW,J_UP,K,N_BETWEEN:INTEGER;
BEGIN
  (* Extend the original VDoS made in MAKE_EXTRAVDOS_POINTS with 19 extra points between
     2 subsequent points by linear interpolation. The original VDoS will be given as
     (X_LARGE[I], G_LARGE[I]) with I=[1..MAX_POINT_LARGE
   *)
  MAKE_LARGE_EXP;
  (* First x[1] in VDoS starts at delta and the 1st box goes from x[1]-delta to
     x[1]+delta and the total width is thus 2*delta
   *)
  X0:=-DELTA;
  FOR I:=1 TO NUMEIN_CLONE DO
  BEGIN
    X_CAL[I]:=X0+2*DELTA; X0:=X_CAL[I];
    (* Get fraction of the left side of the Einstein box and right side of the box: 
       store them in g_left and g_right. One bOX goes from [x0-delta,x0+delta].
       That range defines the endpoints.
     *)
    FOR K:=1 TO 2 DO
    BEGIN
      IF K=1 THEN X_BOUND:=X0-DELTA ELSE X_BOUND:=X0+DELTA;
      IF X_BOUND<0 THEN X_BOUND:=0;
      (* Determine top of the VDoS by finding the nearest neighbours of X_BOUND in the
         original VDoS given as (X_LARGE[1..max_points_large], G_LARGE[1..max_points_large])
       *)
      GET_NEAREST_LARGE(X_BOUND,J_LOW,J_UP);
      (* Get the top by interpolation between the experimental function values of the
         VDoS
       *)
      G:=(G_LARGE[J_UP]-G_LARGE[J_LOW])*(X_BOUND-X_LARGE[J_UP]);
      G:=G/(X_LARGE[J_UP]-X_LARGE[J_LOW])+G_LARGE[J_UP];
      IF G<0 THEN G:=0;
      IF K=1 THEN G_LEFT:=G ELSE G_RIGHT:=G
    END;
    (* We have now 2 points in the frequency box: the endpoints. Next determine all
       points and their number, N, between these endpoints. To do that take the sum of
       all VDoS values. Than add the endpoints and the average is sum+sum_of_endpoints divided
       by N+2. In short: average the VDoS values of the original VDoS in the Einstein
       clone box.
     *)
    GET_BETWEEN_SUM(X0,N_BETWEEN,SUM_BETWEEN);
    (* Add the 2 endpoints x0-delta and x0+delta *)
    N_BETWEEN:=N_BETWEEN+2; SUM_BETWEEN:=SUM_BETWEEN+G_LEFT+G_RIGHT;
    G_CAL[I]:=SUM_BETWEEN/N_BETWEEN
  END;
  (* We refuse zero frequency: if x_cal[1]-delta=0 make it 0.0001 *)
  IF ABS(X_CAL[1]-DELTA)<1e-05 THEN X_CAL[1]:=0.0001+DELTA;
  (* Make the fractions of oscillators in each Einstein box *)
  SUM:=0;
  FOR I:=1 TO NUMEIN_CLONE DO SUM:=SUM+G_CAL[I];
  FOR I:=1 TO NUMEIN_CLONE DO FRAC_WAVE[I]:=G_CAL[I]/SUM
END;

PROCEDURE MAKE_NEW_WAVES;
(* Saves output of MAKE_EINSTEIN_AVERAGE *)
VAR I:INTEGER;
BEGIN
  (* Makes the new Einstein temperatures. *)
  FOR I:=1 TO NUMEIN_CLONE DO
  BEGIN
    (* Einstein monochromators *)
    THETA_WAVE[I]:=X_CAL[I]
  END
END;

PROCEDURE MAKE_SETUP;
(* Make the new VDoS expressed in THETA_WAVE[i], FRAC_WAVE[i].
   We add the 3 acoustic modes and set the acoustic fractions equal to zero later in
   assign_back.
   Note that we make the sum of fractions in the OC modes equal to one always.
 *)
BEGIN
  (* Half-width of an Einstein box is DELTA. The width is the distance between two
     subsequent Einstein frequencies
   *)
  DELTA:=X_EXP[MAX_POINTS]/NUMEIN_CLONE/2;
  (* Make array of mid-points of all Einstein boxes and their heights. In XiPT this
     is the averaging method
   *)
  MAKE_EINSTEIN_AVERAGE;
  (* Einstein boxes defined by mid-point frequency and top (height) as
     X_Cal[1..Numein_clone], G_cal[1..Numein_clone]: Now make setup file
   *)
  MAKE_NEW_WAVES
END;

PROCEDURE ASSIGN_BACK;
(* Assign new wavenumbers and fractions to original 'me' variables. *)
VAR J:INTEGER;
BEGIN
  N_EINSTEIN:=NUMEIN_CLONE;
  (* Set number of modes to selected number NUMEIN_CLONE because the new VDoS has 
     been made with NUMEIN_CLONE modes
   *)
  NUM_FREQ:=NUMEIN_CLONE;
  FOR J:=1 TO NUM_FREQ DO THETA0[J]:=THETA_WAVE[J];
  (* Fractions *)
  FOR J:=1 TO NUM_FREQ DO FRAC[J]:=FRAC_WAVE[J]
END;

PROCEDURE GET_NEAREST_GRUN(X:EXTENDED;NUM_POINTS:INTEGER;VAR J_LOW,J_UP:INTEGER);
(* X is the mid-point of an Einstein box. J_low is the number of the point in the
   original VDoS just below X and J_UP the number of the point just higher as X
 *)
VAR J:INTEGER;STOP_IT:BOOLEAN;
BEGIN
  J:=0; STOP_IT:=FALSE;
  REPEAT
    J:=J+1;
    IF X_GRUN[J]>X THEN STOP_IT:=TRUE
  UNTIL
    (J>=NUM_POINTS) OR (STOP_IT=TRUE);
  J_UP:=J; J_LOW:=J-1;
  IF J_LOW=0 THEN
  BEGIN
    (* In unforseen case when the experimental points start at values above X *)
    J_LOW:=1; J_UP:=2
  END
END;

PROCEDURE GET_BETWEEN_SUM_GRUN(X0:EXTENDED;NUM_POINTS:INTEGER;VAR N_BETWEEN:INTEGER;
          VAR SUM_BETWEEN:EXTENDED);
(* Gets the number of points between <X0-DELTA,X0+DELTA>. Gets the sum of the Gruneisen
   or other parameter values in that interval: sum_between
 *)
VAR X_LEFT,X_RIGHT:EXTENDED;J:INTEGER;
BEGIN
  N_BETWEEN:=0; SUM_BETWEEN:=0;
  X_LEFT:=X0-DELTA; X_RIGHT:=X0+DELTA;
  IF X_LEFT<0 THEN X_LEFT:=0;
  FOR J:=1 TO NUM_POINTS DO
  BEGIN
    IF (X_GRUN[J]>X_LEFT) AND (X_GRUN[J]<X_RIGHT) THEN
    BEGIN
      N_BETWEEN:=N_BETWEEN+1;
      SUM_BETWEEN:=SUM_BETWEEN+G_GRUN[J]
    END
  END
END;

PROCEDURE DISPERSION_AVERAGE;
(* When dispersion is present we average the values at the transition from
   one box having specific gamma,q,a,.. to another box with different gamma,q,a,...
   That can be done in the same way as the averaging method: average all gamma's in the
   new box. This method is an alternative to GRUNEISEN_DISPERSION(P,K,TYP).
   OLD_NUM_FREQ assigned in assign_parameters is the original number of Einstein
   frequencies.
   NUM_FREQ=NUMEIN_CLONE and was assigned in 'assign_back'.
 *)
VAR GAM,GAM_N,X0,G,X_BOUND,SUM_BETWEEN,G_LEFT,G_RIGHT:EXTENDED;
    I,L,N_BLOCK,WHAT,J_LOW,J_UP,N_BETWEEN:INTEGER;
BEGIN
  (* Check if Gruneisen dispersion is present *)
  PHASE_DISPERSE:=FALSE;
  (* # blocks (N_BLOCK) having the same Gruneisen parameters in each block *)
  N_BLOCK:=1;
  GAM:=GAMMA0[1];
  FOR I:=1 TO OLD_NUM_FREQ DO
  BEGIN
    GAM_N:=GAMMA0[I];
    IF ABS(GAM_N-GAM)>2E-6 THEN N_BLOCK:=N_BLOCK+1
  END;
  IF N_BLOCK>1 THEN PHASE_DISPERSE:=TRUE;
  (* Make the new 'me' arrays for Gruneisen, mode-q, etc.. parameters. We already have
     X_EXP[I] as the wavenumbers determined in the original VDoS, but we have added values
     to it that must have fraction 0 in make_extravdos_points. Therefore we make
     X_GRUN and fill it with the original frequencies. OLD_NUM_FREQ is the number of
     frequencies in orginal VDoS.
     Make first Gruneisen or another parameter as function of frequency
   *)
  FOR I:=1 TO OLD_NUM_FREQ DO X_GRUN[I]:=OLD_THETA_WAVE[I];
  FOR WHAT:=1 TO 6 DO
  BEGIN
    FOR I:=1 TO OLD_NUM_FREQ DO
    BEGIN
      (* Assign Gruneisen, or mode-q etc to working array (X_Grun, G_Grun) *)
      CASE WHAT OF
        1:BEGIN
            (* Gruneisen parameters *)
            G_GRUN[I]:=GAMMA0[I]
          END;
        2:BEGIN
            (* Mode-q parameters *)
            G_GRUN[I]:=Q0[I]
          END;
        3:BEGIN
            (* Mode-q1 parameters *)
            G_GRUN[I]:=GAMMA_INF[I]
          END;
        4:BEGIN
            (* Mode-q2 or gamma at infinite pressure parameters: NOT PRESENT *)
            G_GRUN[I]:=0
          END;
        5:BEGIN
            (* Mode anharmonicity parameters *)
            G_GRUN[I]:=ANH_A0[I]
          END;
        6:BEGIN
            (* Mode anharmonicity parameters *)
            G_GRUN[I]:=ANH_Z0[I]
          END;
      END
    END;
    (* First x[1] in VDoS starts at delta and the 1st box goes from x[1]-delta to
       x[1]+delta and the total width is 2*delta. Find parameters, by interpolation, in
       the same way as in MAKE_EINSTEIN_AVERAGE for VDoS fractions
     *)
    X0:=-DELTA;
    FOR I:=1 TO NUMEIN_CLONE DO
    BEGIN
      X_CAL[I]:=X0+2*DELTA; X0:=X_CAL[I];
      (* Get Gruneisen or other parameter of the left side of the block and right side
         of the block: store them in g_left and g_right. One block goes from
         [x0-delta,x0+delta]. That are the endpoints.
       *)
      FOR L:=1 TO 2 DO
      BEGIN
        IF L=1 THEN X_BOUND:=X0-DELTA ELSE X_BOUND:=X0+DELTA;
        IF X_BOUND<0 THEN X_BOUND:=0;
        (* Determine top of the parameter by finding the nearest neighbours of X_BOUND in the
           original parameter array given in (X_GRUN[1..old_num_freq], G_GRUN[1..old_num_freq])
         *)
        GET_NEAREST_GRUN(X_BOUND,OLD_NUM_FREQ,J_LOW,J_UP);
        (* Get the top by interpolation between the original parameter values of the
           original database
         *)
        G:=(G_GRUN[J_UP]-G_GRUN[J_LOW])*(X_BOUND-X_GRUN[J_UP]);
        G:=G/(X_GRUN[J_UP]-X_GRUN[J_LOW])+G_GRUN[J_UP];
        IF L=1 THEN G_LEFT:=G ELSE G_RIGHT:=G
      END;
      (* We have now 2 points in the parameter block: the endpoints. Next determine all
         points and their number, N, between these endpoints. To do that take the sum of all
         parameter values. Than add the endpoints and the average is sum+sum_of_endpoints divided
         by N+2
       *)
      GET_BETWEEN_SUM_GRUN(X0,OLD_NUM_FREQ,N_BETWEEN,SUM_BETWEEN);
      (* Add the 2 endpoints x0-delta and x0+delta *)
      N_BETWEEN:=N_BETWEEN+2; SUM_BETWEEN:=SUM_BETWEEN+G_LEFT+G_RIGHT;
      G_CAL[I]:=SUM_BETWEEN/N_BETWEEN;
    END;
    (* Now we have (X_cal[i], g_cal[i]) and we need only g_cal[i], which is either
       Gruneisen parameter, mode-q, etc
     *)
    FOR I:=1 TO NUMEIN_CLONE DO
    BEGIN
      CASE WHAT OF
        1:BEGIN
            (* Gruneisen parameters *)
            GAMMA0[I]:=G_CAL[I]
          END;
        2:BEGIN
            (* Mode-q parameters *)
            Q0[I]:=G_CAL[I]
          END;
        3:BEGIN
            (* Mode-q1 parameters *)
            GAMMA_INF[I]:=G_CAL[I]
          END;
        4:BEGIN
            (* Mode-q2 or gamma at infinite pressure parameters: IS NOT PRESENT *)
          END;
        5:BEGIN
            (* Mode anharmonicity parameters *)
            ANH_A0[I]:=G_CAL[I]
          END;
        6:BEGIN
            (* Mode anharmonicity parameters *)
            ANH_Z0[I]:=G_CAL[I]
          END
      END
    END
  END
END;

PROCEDURE SET_START_VDOS;
(* We must make multiplications with this spectrum *)
VAR J,NUM:INTEGER;
BEGIN
  NUM:=NUM_FREQ;
  FOR J:=1 TO NUM DO WAVE_CONVERT[J]:=THETA0[J]
END;


PROCEDURE TROUBLE_F(VAR FAC:EXTENDED);
(* Factor must be larger than zero *)
BEGIN
  IF FAC<1E-3 THEN FAC:=1E-3
END;

PROCEDURE CHECK_FAC(VAR CO:INTEGER;VAR DF:EXTENDED;VAR LONG_F:BOOLEAN);
(* If after CO steps in the repeat-until scan in factor not a minimum has been
   achieved, increase the step with a factor 3
 *)
BEGIN
  IF CO>8 THEN
  BEGIN
    IF ABS(DF)<1E-3 THEN
    BEGIN
      DF:=DF*3; CO:=0; LONG_F:=TRUE
    END
  END;
  IF CO>30000 THEN CO:=0
END;

PROCEDURE MAKE_FREQUENCIES(FAC:EXTENDED);
(* Multiply all frequencies with factor FAC. NUM_FREQ has been assigned the
   value NUMEIN_CLONE in ASSIGN_BACK(). 
   Frequencies FT_MIX for endmembers and FT_COM for compounds. The first three
   are acoustic and will not be changed. Because their fractions are zero they
   do not contribute to the VDoS. We must make multiplications with factor FAC
   of the same spectrum given in wave_convert[j] made in SET_START_VDOS(). That
   spectrum remains the same, only FAC changes
 *)
VAR J,NUM:INTEGER;
BEGIN
  NUM:=NUM_FREQ;
  FOR J:=1 TO NUM DO THETA0[J]:=FAC*WAVE_CONVERT[J]
END;

FUNCTION ENTROPY_DIF(FAC:EXTENDED):EXTENDED;
(* Calculates absolute difference between entropy and the target value S_TARGET.
   S_TARGET is the entropy at target conditions T_CLONE,P_TCLONE, eg 298.15 K and 1 bar.
 *)
VAR V0_ST_FIX,VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CV,CP,BULKMOD,DPDT,ALPHA,
    ZERO_PNT:EXTENDED;
    FIX_YES:BOOLEAN;
BEGIN
  MAKE_FREQUENCIES(FAC);
  V0_ST_FIX:=V0_ST; FIX_YES:=FALSE; FIX_STATIC_VOLUME(V0_ST_FIX,FIX_YES);
  V0_ST:=V0_ST_FIX;
  (* Calculate thermodynamic properties *)
  GIBBS:=0; HELMHOLTZ:=0; ENTHALPY:=0; ENERGY:=0; ENTROPY:=0; CP:=0; CV:=0;
  BULKMOD:=0; DPDT:=0; ALPHA:=0; ZERO_PNT:=0; VPT:=V0;
  GET_THERMO_PROPERTIES(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,
                        BULKMOD,DPDT,ALPHA,ZERO_PNT);
  ENTROPY_DIF:=ABS(ENTROPY-S_TARGET)
END;

PROCEDURE STRETCH_VDOS(VAR STRETCH_FACTOR,DV_STATIC:EXTENDED);
(* Stretch/contract the VDoS such that the entropy is the same as calculated
   from the original VDoS. Volume at (0K,0Pa) must be kept the same. S_TARGET must be
   kept the same by targetting it
 *)
VAR FAC,DF,DF1,FAC_M,FAC_LAST,S_LAST,S_NEW,V0_ST_FIX:EXTENDED;
    COUNTER:INTEGER;
    LONG_F,SW,FIX_YES:BOOLEAN;
BEGIN
  SET_START_VDOS;
  (* Set target conditions e.g. 298.15 K and 1 bar *)
  TEMPERATURE:=T_CLONE; PRESSURE:=P_CLONE;
  FAC:=1; DF:=0.001; LONG_F:=FALSE; SW:=FALSE; COUNTER:=0;
  WHILE ABS(DF)>1E-8 DO
  BEGIN
    DF1:=DF;
    REPEAT
      TROUBLE_F(FAC); FAC_M:=FAC; COUNTER:=0;
      IF LONG_F=TRUE THEN DF:=DF/3;
      IF (LONG_F=TRUE) AND (ABS(DF)<=ABS(DF1)) THEN LONG_F:=FALSE;
      IF SW=FALSE THEN S_LAST:=ENTROPY_DIF(FAC);
      FAC:=FAC+DF; TROUBLE_F(FAC); SW:=TRUE;
      S_NEW:=ENTROPY_DIF(FAC);
      FAC_LAST:=FAC;
      IF S_NEW>=S_LAST THEN
      BEGIN
        DF:=-DF; FAC:=FAC_M; S_NEW:=S_LAST
      END;
      REPEAT
        S_LAST:=S_NEW; FAC_LAST:=FAC; FAC:=FAC+DF; TROUBLE_F(FAC);
        COUNTER:=COUNTER+1;
        S_NEW:=ENTROPY_DIF(FAC);
        CHECK_FAC(COUNTER,DF,LONG_F);
      UNTIL
        (S_NEW>=S_LAST);
      FAC:=FAC_LAST
    UNTIL
      (LONG_F=FALSE);
    DF:=ABS(DF)/3
  END;
  STRETCH_FACTOR:=FAC;
  (* Make the final frequencies *)
  MAKE_FREQUENCIES(FAC);
  V0_ST_FIX:=V0_ST; FIX_YES:=FALSE; FIX_STATIC_VOLUME(V0_ST_FIX,FIX_YES);
  V0_ST:=V0_ST_FIX;
  DV_STATIC:=V0_ST-OLD_STATIC_VOLUME
END;

PROCEDURE CORRECT_ENTHALPY;
(* Correct for enthalpy at target conditions e.g. 1 bar and 298.15 K *)
VAR VPT,DH,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CV,CP,BULKMOD,DPDT,ALPHA,
    ZERO_PNT:EXTENDED;
BEGIN
  (* Set target conditions, e.g. 298.15 K and 1 bar *)
  TEMPERATURE:=T_CLONE; PRESSURE:=P_CLONE;
  VPT:=V0;
  GIBBS:=0; HELMHOLTZ:=0; ENTHALPY:=0; ENERGY:=0; ENTROPY:=0; CP:=0; CV:=0;
  BULKMOD:=0; DPDT:=0; ALPHA:=0; ZERO_PNT:=0;
  GET_THERMO_PROPERTIES(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,
                        BULKMOD,DPDT,ALPHA,ZERO_PNT);
  DH:=ENTHALPY-H_TARGET;
  (* Correct the reference energy such that the enthalpy at the target condition
     remains the same as calculated from the orginal VDoS
   *)
  U_REF:=U_REF-DH;
  GET_THERMO_PROPERTIES(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,
                        BULKMOD,DPDT,ALPHA,ZERO_PNT);
  H_TARGET_CAL:=ENTHALPY; S_TARGET_CAL:=ENTROPY; V_TARGET_CAL:=VPT
END;

PROCEDURE MAKE_NEW_DATABASE;
(* Makes new database with specified number of Einstein frequencies NUMEIN_CLONE.
   This procedure is the main procedure treating a pure substance.
   BY ASSIGNING STRETCHES REMEMBER THAT FRAC[1]->FRAC_KIEF^[K,P,4], SAME FOR FREQUENCY
   STRETCH_FACTOR is the stretch factor to multiply the frequencies in the VDoS,
   DV_STATIC is the difference in static volum resulting from the new VDoS and original
    VDoS
 *)
VAR STRETCH_FACTOR,DV_STATIC:EXTENDED;
    V0_ST_FIX:EXTENDED;
    DONE_IT,FIX_YES:BOOLEAN;
BEGIN
  (* Initialize volumes VMEM *);
  VMEMORY:=V0; OLD_VMEMORY:=V0;
  (* Initializes array indicating that dispersion in Gruneisen parameters is present *)
  INITIALIZE_DISPERSION;
  (* Set target condition, e.g 298.15 K and 1 bar *)
  TEMPERATURE:=T_CLONE; PRESSURE:=P_CLONE;
  DONE_IT:=FALSE;
  (* Only Kieffer model. Make new VDoS *)
  TARGET_PROPERTIES; ASSIGN_PARAMETERS; DONE_IT:=TRUE;
  MAKE_EXTRAVDOS_POINTS; MAKE_SETUP; ASSIGN_BACK;
  (* Take into account that Gruneisen parameters may be disperse *)
  DISPERSION_AVERAGE;
  (* Correct static lattice volume *)
  V0_ST_FIX:=V0_ST; FIX_YES:=FALSE; FIX_STATIC_VOLUME(V0_ST_FIX,FIX_YES);
  V0_ST:=V0_ST_FIX; STRETCH_FACTOR:=1; DV_STATIC:=0;
  (* Correct VDoS by using targeting entropy *)
  STRETCH_VDOS(STRETCH_FACTOR,DV_STATIC);
  (* Write stretch_factor and difference in static volume 
     WRITE(STRETCH_FACTOR:14:9,'   ',DV_STATIC*1E6:14:9);
   *)
  (* Correct for H298 or target enthalpy and calculate final result *)
  CORRECT_ENTHALPY;
  IF DONE_IT=TRUE THEN
  BEGIN
    SAVE_RESULT_FILE(OLD_NUM_FREQ,OLD_STATIC_VOLUME,H_TARGET,H_TARGET_CAL,S_TARGET,
                     S_TARGET_CAL,V_TARGET,V_TARGET_CAL,STRETCH_FACTOR,PHASE_DISPERSE)
  END
END;


PROCEDURE CONVERT_VDOS;
VAR OLD_EXTRAPOLATE:BOOLEAN;
BEGIN
  OLD_EXTRAPOLATE:=EXTRAPOLATE; EXTRAPOLATE:=FALSE;
  MAKE_NEW_DATABASE;
  EXTRAPOLATE:=OLD_EXTRAPOLATE
END;

END.
