UNIT DEFECTS;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Contributions to properties due to lattice defects in the
   multiple Einstein method. See manual chapter 16
   Defects are treated as mono vacancies with the model given by:
   Dorogokupets & Oganov 2007 Phys Rev B 75:024115

   Helmholtz energy contribution: A=-(3/2)nRT*exp(s*eta^f-h*eta^g/T)
   DEFECT[1]: h: enthalpy, unit Kelvin
   DEFECT[2]: s: entropy,  no unit
   DEFECT[3]: f: no unit
   DEFECT[4]: g: no unit
   eta=V/V0, n=# atoms per formula unit
 *)

INTERFACE
USES CRT,sysutils,variables;


FUNCTION DEFECT_PRESSURE(VPT:EXTENDED):EXTENDED;
PROCEDURE MONOVACANCY(VPT:EXTENDED;VAR U_DEF,S_DEF,CV_DEF,K_DEF,DPDT_DEF:EXTENDED);
IMPLEMENTATION

FUNCTION DEFECT_PRESSURE(VPT:EXTENDED):EXTENDED;
(* Pressure contributon for the formation of monovacancies. H,S are the heat and entropy 
   of vacancy formation. Unit of H is Kelvin and S is dimensionless. Eqn. 17.4
 *)
VAR ETA,H,S,F,G,Z,Z1:EXTENDED;
BEGIN
  IF DEFECT_YES>0 THEN
  BEGIN
    IF TEMPERATURE>1E-3 THEN
    BEGIN
      H:=DEFECT[1]; S:=DEFECT[2];
      F:=DEFECT[3]; G:=DEFECT[4];
      ETA:=VPT/V0;
      Z:=S*POW_N(ETA,F)-H*POW_N(ETA,G)/TEMPERATURE; Z:=EXP(Z);
      Z1:=S*F*POW_N(ETA,F)-H*G*POW_N(ETA,G)/TEMPERATURE;
      DEFECT_PRESSURE:=1.5*N_MOL*GAS_CONSTANT*TEMPERATURE*Z*Z1/VPT
    END
    ELSE DEFECT_PRESSURE:=0
  END
  ELSE DEFECT_PRESSURE:=0
END;

FUNCTION A_DEFECT(VPT:EXTENDED):EXTENDED;
(* Helmholtz energy for the formation of monovacancies. H,S are the heat and entropy 
   of formation in Kelvin and S is dimensionless. Eqn. 17.1
 *)
VAR ETA,H,S,F,G,Z:EXTENDED;
BEGIN
  IF TEMPERATURE>1E-3 THEN
  BEGIN
    H:=DEFECT[1]; S:=DEFECT[2];
    F:=DEFECT[3]; G:=DEFECT[4];
    ETA:=VPT/V0;
    Z:=S*POW_N(ETA,F)-H*POW_N(ETA,G)/TEMPERATURE; Z:=EXP(Z);
    A_DEFECT:=-1.5*N_MOL*GAS_CONSTANT*TEMPERATURE*Z
  END
  ELSE A_DEFECT:=0
END;

FUNCTION S_DEFECT(VPT:EXTENDED):EXTENDED;
(* Entropy for the formation of monovacancies. H,S are the heat and entropy 
   of formation in Kelvin and S is dimensionless. Eqn. 17.2
 *)
VAR ETA,H,S,F,G,Z:EXTENDED;
BEGIN
  IF TEMPERATURE>1E-3 THEN
  BEGIN
    H:=DEFECT[1]; S:=DEFECT[2];
    F:=DEFECT[3]; G:=DEFECT[4];
    ETA:=VPT/V0;
    Z:=S*POW_N(ETA,F)-H*POW_N(ETA,G)/TEMPERATURE; Z:=EXP(Z);
    S_DEFECT:=1.5*N_MOL*GAS_CONSTANT*Z*(1+H*POW_N(ETA,G)/TEMPERATURE)
  END
  ELSE S_DEFECT:=0
END;

FUNCTION CV_DEFECT(VPT:EXTENDED):EXTENDED;
(* Heat capacity, Cv for the formation of monovacancies. H,S are the heat and entropy 
   of formation in Kelvin and S is dimensionless. Eqn. 17.3
 *)
VAR ETA,H,S,F,G,Z,Z1:EXTENDED;
BEGIN
  IF TEMPERATURE>1E-3 THEN
  BEGIN
    H:=DEFECT[1]; S:=DEFECT[2];
    F:=DEFECT[3]; G:=DEFECT[4];
    ETA:=VPT/V0;
    Z:=S*POW_N(ETA,F)-H*POW_N(ETA,G)/TEMPERATURE; Z:=EXP(Z);
    Z1:=H*POW_N(ETA,G)/TEMPERATURE;
    CV_DEFECT:=1.5*N_MOL*GAS_CONSTANT*Z*Z1*Z1
  END
  ELSE CV_DEFECT:=0
END;

FUNCTION DPDT_DEFECT(VPT:EXTENDED):EXTENDED;
(* Temperature derivative of pressure contributon for the formation of monovacancies
   taken at constant volume. H,S are the heat and entropy of formation in Kelvin and
   S is dimensionless. Eqn. 17.5
 *)
VAR ETA,H,S,F,G,SQUARE_TEMPERATURE,Z,Z1:EXTENDED;
BEGIN
  IF TEMPERATURE>1E-3 THEN
  BEGIN
    H:=DEFECT[1]; S:=DEFECT[2];
    F:=DEFECT[3]; G:=DEFECT[4];
    ETA:=VPT/V0;
    Z:=S*POW_N(ETA,F)-H*POW_N(ETA,G)/TEMPERATURE; Z:=EXP(Z);
    Z1:=S*F*POW_N(ETA,F);
    SQUARE_TEMPERATURE:=TEMPERATURE*TEMPERATURE;
    Z1:=Z1+H*POW_N(ETA,G)*(S*F*POW_N(ETA,F)/TEMPERATURE-H*G*POW_N(ETA,G)/SQUARE_TEMPERATURE);
    DPDT_DEFECT:=1.5*N_MOL*GAS_CONSTANT*Z*Z1/VPT
  END
  ELSE DPDT_DEFECT:=0
END;

FUNCTION K_DEFECT(VPT:EXTENDED):EXTENDED;
(* Isothermal bulk modulus contributon for the formation of monovacancies.
   H,S are the heat and entropy of formation in Kelvin and S is dimensionless.
   Eqn. 17.6
 *)
VAR ETA,H,S,F,G,Z,Z1,Z2,F1,G1:EXTENDED;
BEGIN
  IF TEMPERATURE>1E-3 THEN
  BEGIN
    H:=DEFECT[1]; S:=DEFECT[2];
    F:=DEFECT[3]; G:=DEFECT[4];
    ETA:=VPT/V0;
    Z:=S*POW_N(ETA,F)-H*POW_N(ETA,G)/TEMPERATURE; Z:=EXP(Z);
    F1:=F-1; G1:=G-1;
    Z1:=S*F*F1*POW_N(ETA,F1)-H*G*G1*POW_N(ETA,G1)/TEMPERATURE;
    Z2:=S*F*POW_N(ETA,F1)-H*G*POW_N(ETA,G1)/TEMPERATURE;
    Z2:=Z2*Z2*ETA;
    Z1:=Z1+Z2;
    K_DEFECT:=-1.5*N_MOL*GAS_CONSTANT*TEMPERATURE*Z*Z1/V0
  END
  ELSE K_DEFECT:=0
END;

PROCEDURE MONOVACANCY(VPT:EXTENDED;VAR U_DEF,S_DEF,CV_DEF,K_DEF,DPDT_DEF:EXTENDED);
(* Calculation of contributions due to defects in formalism of Dorogokupets and
   Oganov (2007), Phys Rev B 75:024115
 *)
VAR A_DEF:EXTENDED;
BEGIN
  (* Initialize variables *)
  U_DEF:=0; S_DEF:=0; CV_DEF:=0; K_DEF:=0; DPDT_DEF:=0;
  IF DEFECT_YES>0 THEN
  BEGIN
    A_DEF:=A_DEFECT(VPT);
    S_DEF:=S_DEFECT(VPT);
    U_DEF:=A_DEF+TEMPERATURE*S_DEF;
    CV_DEF:=CV_DEFECT(VPT);
    K_DEF:=K_DEFECT(VPT);
    DPDT_DEF:=DPDT_DEFECT(VPT)
  END
END;

END.
