UNIT magnetic;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Unit for calculating magnetic properties from Inden's (1981) model:
   Inden (1981), The role of magnetism in the calculation of phase diagrams. Physica,
   103B:82-100.
   This model is expanded in MacLaurin series. The magnetic entropy contribution at 0 K is
   zero and obeys the 3rd law.
   The number of MacLaurin terms in the summation is defined by the user and stored
   in variable NUM_MCLAURIN_TERMS. The summations always start with j=0.
   In Calphad only 3 terms are defined: j=0,1,2. In this case the last
   term is NUM_MCLAURIN_TERMS-1
   See manual chapter 11
 *)
INTERFACE
USES CRT,sysutils,variables;


PROCEDURE CONVERT_INDEN_A1A2(PFAC,M,N:EXTENDED;VAR A1,A2:EXTENDED);
PROCEDURE CONVERT_MAGNETICS;
PROCEDURE CONVERT_INDEN_PFAC;
PROCEDURE TC_AND_BETA(VAR TC,BM,A1,M,A2,N:EXTENDED);
FUNCTION A_LAMBDA:EXTENDED;
FUNCTION G_LAMBDA:EXTENDED;
FUNCTION S_LAMBDA:EXTENDED;
FUNCTION CV_LAMBDA:EXTENDED;
PROCEDURE GET_MAGNETIC_PROPERTIES(VAR A_MAG,U_MAG,S_MAG,CV_MAG:EXTENDED);

IMPLEMENTATION


FUNCTION SERIES_J:EXTENDED;
(* Series required for calculation of a1,a2 in the Inden magnetic model. See
   me-freepascal-manual.doc, eqn (11.10, 11.11)
 *)
VAR J,FAC:INTEGER;
    SUM:EXTENDED;
BEGIN
  FAC:=1; SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC:=2*J+1; FAC:=FAC*FAC;
    SUM:=SUM+1/FAC
  END;
  SERIES_J:=SUM
END;

FUNCTION SERIES_S1(M:EXTENDED):EXTENDED;
(* Series S1 required for calculation of a1,a2 in the Inden magnetic model. See
   me-freepascal-manual.doc, eqn (11.12)
 *)
VAR J,FAC:INTEGER;
    SUM,FAC_M:EXTENDED;
BEGIN
  FAC:=1; SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC:=2*J+1; FAC_M:=FAC*(FAC*M+1);
    SUM:=SUM+1/FAC_M
  END;
  SERIES_S1:=SUM
END;

FUNCTION SERIES_S2(N:EXTENDED):EXTENDED;
(* Series S1 required for calculation of a1,a2 in the Inden magnetic model. See
   me-freepascal-manual.doc, eqn (11.13)
 *)
VAR J,FAC:INTEGER;
    SUM,FAC_N:EXTENDED;
BEGIN
  FAC:=1; SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC:=2*J+1; FAC_N:=FAC*(-FAC*N+1);
    SUM:=SUM+1/FAC_N
  END;
  SERIES_S2:=SUM
END;

PROCEDURE CONVERT_INDEN_A1A2(PFAC,M,N:EXTENDED;VAR A1,A2:EXTENDED);
(* Inden model: parameters (p,m,n) are converted to (m,a1,n,a2): see eqn.
   (11.10) and (11.11) in me-freepascal-manual.doc. We enter with p=magnetic p-factor,
   m and n in Inden's (1981) model and return a1,a2.
   I rewrite the formulas such that we do not divide by zero when pfac=0.
 *)
VAR S1,S2,SJ,Z1,Z2:EXTENDED;
BEGIN
  IF PFAC>1E-11 THEN
  BEGIN
    S1:=SERIES_S1(M); S2:=SERIES_S2(N); SJ:=SERIES_J;
    (* Convert (m,n,p) to (m,n,a1) *)
    Z1:=S2/S1; Z1:=Z1*(PFAC-1);
    Z2:=Z1*N+M*PFAC;
    A1:=M*(1-M*PFAC/Z2)/SJ/2;
    (* Convert (m,n,p) to (m,n,a2) *)
    Z2:=Z1+M*PFAC/N;
    A2:=M*PFAC/Z2/SJ/2
  END
  ELSE
  BEGIN
    A1:=0; A2:=0
  END
END;

PROCEDURE CONVERT_MAGNETICS;
(* In the case that magnetic phases are described with the Inden (1981), Physica,
   103B, 82-100 model. Convert (N,M,P)=(INDEN_M,INDEN_N,INDEN_P) to
   (M,A1,N,A2) that gives simpler formulas and avoids divisions by zero. This procedure
   is important for determining A1, A2 calculated by series expansions. That is done
   only once instead of doing this at every (Pressure,Temperature) step to increase
   the speed of the calculations
 *)
VAR A1,A2:EXTENDED;
BEGIN
  IF MAGNETIC_YES>0 THEN
  BEGIN
    CONVERT_INDEN_A1A2(INDEN_P,INDEN_M,INDEN_N,A1,A2);
    INDEN_A1:=A1; INDEN_A2:=A2
  END
  ELSE
  BEGIN
    INDEN_A1:=0; INDEN_A2:=0
  END
END;

PROCEDURE CONVERT_INDEN_PFAC;
(* Inden model: parameters (m,a1,n,a2) are converted to (m,n,p): see eqn.
   (11.10) and (11.11) in me-freepascal-manual.doc.
    PFAC is the variable p in Inden's (1981) model.
 *)
VAR M,N,S1,S2,PFAC:EXTENDED;
BEGIN
  PFAC:=INDEN_P; 
  IF PFAC>1E-11 THEN
  BEGIN
    M:=INDEN_M; N:=INDEN_N;
    S1:=SERIES_S1(M); S2:=SERIES_S2(N);
    PFAC:=-INDEN_A2*S2/(INDEN_A1*S1-INDEN_A2*S2)
  END
END;

PROCEDURE TC_AND_BETA(VAR TC,BM,A1,M,A2,N:EXTENDED);
(* Curie temperature, magnetic moment, A1,A2 in Cv expression (11.9) *)
BEGIN
  TC:=T_CURIE;
  BM:=MAG_MOMENT;
  (* Inden's model for magnetism *)
  A1:=INDEN_A1; M:=INDEN_M; A2:=INDEN_A2; N:=INDEN_N
END;

FUNCTION SERIES_GTAU_M(TAU,M:EXTENDED):EXTENDED;
(* Inden's model, see me-freepascal-manual.doc eqn. (11.6): series in G_Tau: 1st term
   containing a1: tau<=1
 *)
VAR J,FAC_J:INTEGER;
    FAC_M,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_M:=FAC_J*M+1;
    EXPON:=FAC_J*M; Z:=POW_N(TAU,EXPON)-1/TAU;
    SUM:=SUM+Z/FAC_J/FAC_J/FAC_M
  END;
  SERIES_GTAU_M:=SUM
END;

FUNCTION SERIES_SQUARE_N(TAU,N:EXTENDED):EXTENDED;
(* Inden's model, see me-freepascal-manual.doc eqn. (11.6): series in G_Tau: 2nd term
   with a2: tau<=1
 *)
VAR J,FAC_J:INTEGER;FAC_N,SUM:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_N:=-FAC_J*N+1;
    SUM:=SUM+1/FAC_J/FAC_J/FAC_N
  END;
  SERIES_SQUARE_N:=SUM
END;

FUNCTION SERIES_GTAU_N(TAU,N:EXTENDED):EXTENDED;
(* Inden's model, see me-freepascal-manual.doc eqn. (11.7): series in G_Tau: 1st term
   with a2: tau>1
 *)
VAR J,FAC_J:INTEGER;FAC_N,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_N:=-FAC_J*N+1;
    EXPON:=-FAC_J*N; Z:=POW_N(TAU,EXPON);
    SUM:=SUM+Z/FAC_J/FAC_J/FAC_N
  END;
  SERIES_GTAU_N:=SUM
END;

FUNCTION G_TAU(TAU,A1,M,A2,N:EXTENDED):EXTENDED;
(* G(Tau), eqn. (11.6) and (11.7) see me-freepascal-manual.doc *)
BEGIN
  (* Inden's magnetic model using NUM_MCLAURIN_TERMS in the series expansion for Cv *)
  IF TAU<=1 THEN
  BEGIN
    G_TAU:=-2*A1*SERIES_GTAU_M(TAU,M)/M-1/TAU+2*A2*SERIES_SQUARE_N(TAU,N)/N/TAU
  END
  ELSE
  BEGIN
    (* At tau->infinity is g_tau=-1 *)
    IF TAU<1E6 THEN
    BEGIN
      G_TAU:=-1+2*A2*SERIES_GTAU_N(TAU,N)/N
    END
    ELSE G_TAU:=-1
  END   
END;

FUNCTION SERIES_DGDTAU_M(TAU,M:EXTENDED):EXTENDED;
(* Inden's model, see me-freepascal-manual.doc eqn. (11.15): series in DG_DTau: 1st term
   with a1: tau<=1
 *)
VAR J,FAC_J:INTEGER;FAC_M,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_M:=FAC_J*M+1;
    EXPON:=FAC_J*M-1; Z:=POW_N(TAU,EXPON);
    SUM:=SUM+Z/FAC_J/FAC_M
  END;
  SERIES_DGDTAU_M:=SUM
END;

FUNCTION SERIES_SQUARE_M(TAU,M:EXTENDED):EXTENDED;
(* Inden's model, see me-freepascal-manual.doc eqn. (11.15): series in G_Tau: 2nd term
   with a1: tau<=1
 *)
VAR J,FAC_J:INTEGER;FAC_M,SUM:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_M:=FAC_J*M+1;
    SUM:=SUM+1/FAC_J/FAC_J/FAC_M
  END;
  SERIES_SQUARE_M:=SUM
END;

FUNCTION SERIES_DGDTAU_N(TAU,N:EXTENDED):EXTENDED;
(* Inden's model, see me-freepascal-manual.doc eqn. (11.16): series in DG_DTau: 1st term
   with a2: tau>1
 *)
VAR J,FAC_J:INTEGER;FAC_N,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_N:=-FAC_J*N+1;
    EXPON:=-FAC_J*N-1; Z:=POW_N(TAU,EXPON);
    SUM:=SUM+Z/FAC_J/FAC_N
  END;
  SERIES_DGDTAU_N:=SUM
END;

FUNCTION DG_DTAU(TAU,A1,M,A2,N:EXTENDED):EXTENDED;
(* Inden's formalism of magnetism uses a1,m,a2,n: eqn. (11.15,11.16) see me-freepascal-manual.doc
   Used in volume dependent magnetism to derive P_lambda, K_lambda
 *)
VAR A:EXTENDED;
BEGIN
  (* Inden model *)
  IF TAU<=1 THEN
  BEGIN
    A:=-2*A1*SERIES_DGDTAU_M(TAU,M)-2*A1*SERIES_SQUARE_M(TAU,M)/M/TAU/TAU;
    A:=A+1/TAU/TAU;
    DG_DTAU:=A-2*A2*SERIES_SQUARE_N(TAU,N)/N/TAU/TAU
  END
  ELSE
  BEGIN
    (* At tau->infinity => dg/dtau->0 *)
    IF TAU<1E6 THEN
    BEGIN
      DG_DTAU:=-2*A2*SERIES_DGDTAU_N(TAU,N)
    END
    ELSE DG_DTAU:=0
  END
END;

FUNCTION SERIES_D2GDTAU_M(TAU,M:EXTENDED):EXTENDED;
(* Inden's model, see me-freepascal-manual.doc eqn. (11.17): series in D2G_DTau: 1st term
   with a1: tau<=1
 *)
VAR J,FAC_J:INTEGER;FAC_M,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_M:=FAC_J*M;
    EXPON:=FAC_M-2; Z:=POW_N(TAU,EXPON);
    SUM:=SUM+Z*(FAC_M-1)/FAC_J/(FAC_M+1)
  END;
  SERIES_D2GDTAU_M:=SUM
END;

FUNCTION SERIES_D2GDTAU_N(TAU,N:EXTENDED):EXTENDED;
(* Inden's model, see magnetic.doc eqn. (11.18): series in D2G_DTau: 1st term
   with a2: tau>1
 *)
VAR J,FAC_J:INTEGER;FAC_N,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
  BEGIN
    FAC_J:=2*J+1; FAC_N:=-FAC_J*N;
    EXPON:=FAC_N-2; Z:=POW_N(TAU,EXPON);
    SUM:=SUM+Z*(FAC_N-1)/FAC_J/(FAC_N+1)
  END;
  SERIES_D2GDTAU_N:=SUM
END;

FUNCTION D2G_DTAU(P,TYP:INTEGER;TAU,A1,M,A2,N:EXTENDED):EXTENDED;
(* Inden's formalism of magnetism uses a1,m,a2,n: eqn. (11.17,11.18) see me-freepascal-manual.doc
 *)
VAR A:EXTENDED;
BEGIN
  (* Inden's model *)
  IF TAU<=1 THEN
  BEGIN
    A:=-2*A1*SERIES_D2GDTAU_M(TAU,M)+4*A1*SERIES_SQUARE_M(TAU,M)/M/TAU/TAU/TAU;
    A:=A-2/TAU/TAU/TAU;
    D2G_DTAU:=A+4*A2*SERIES_SQUARE_N(TAU,N)/N/TAU/TAU/TAU
  END
  ELSE
  BEGIN
    (* At tau->infinity => d2/dtau2->0 *)
    IF TAU<1E6 THEN
    BEGIN
      D2G_DTAU:=-2*A2*SERIES_D2GDTAU_N(TAU,N)
    END
    ELSE D2G_DTAU:=0
  END
END;

FUNCTION SERIES_ZC(TAU,A1,M,A2,N:EXTENDED):EXTENDED;
(* Inden's formalism of magnetism uses a1,m,a2,n: see me-freepascal-manual.doc
   This function is 2*dg/dt+t*d2g/dt2 appearing in Cv with t=tau. Eqn.(11.19)+(11.20).
   This series is zero at tau=0 and at tau=infinite.
 *)
VAR J,FAC_J:INTEGER;FAC_M,FAC_N,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  IF TAU<=1 THEN
  BEGIN
    IF TAU>1E-3 THEN
    BEGIN
      FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
      BEGIN
        FAC_J:=2*J+1; FAC_M:=FAC_J*M;
        EXPON:=FAC_M-1; Z:=POW_N(TAU,EXPON);
        SUM:=SUM+Z/FAC_J
      END;
      SERIES_ZC:=-2*A1*SUM
    END
    ELSE SERIES_ZC:=0
  END
  ELSE
  BEGIN
    IF TAU<1E6 THEN
    BEGIN
      FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
      BEGIN
        FAC_J:=2*J+1; FAC_N:=-FAC_J*N;
        EXPON:=FAC_N-1; Z:=POW_N(TAU,EXPON);
        SUM:=SUM+Z/FAC_J
      END;
      SERIES_ZC:=-2*A2*SUM
    END
    ELSE SERIES_ZC:=0
  END
END;

FUNCTION SERIES_ZS(TAU,A1,M,A2,N:EXTENDED):EXTENDED;
(* Inden's formalism of magnetism uses a1,m,a2,n: see me-freepascal-manual.doc
   This function is g(t)+t*dg/dt appearing in entropy with t=tau. Eqn.(11.21)+(11.22).
   This series is 0 at tau=0 and it is -1 at tau=infinite.
 *)
VAR J,FAC_J:INTEGER;FAC_M,FAC_N,SUM,EXPON,Z:EXTENDED;
BEGIN
  SUM:=0;
  IF TAU<=1 THEN
  BEGIN
    IF TAU>1E-3 THEN
    BEGIN
      FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
      BEGIN
        FAC_J:=2*J+1; FAC_M:=FAC_J*M;
        EXPON:=FAC_M; Z:=POW_N(TAU,EXPON);
        SUM:=SUM+Z/FAC_J/FAC_J
      END;
      SERIES_ZS:=-2*A1*SUM/M
    END
    ELSE SERIES_ZS:=0
  END
  ELSE
  BEGIN
    IF TAU<1E6 THEN
    BEGIN
      FOR J:=0 TO NUM_MCLAURIN_TERMS-1 DO
      BEGIN
        FAC_J:=2*J+1; FAC_N:=-FAC_J*N;
        EXPON:=FAC_N; Z:=POW_N(TAU,EXPON);
        SUM:=SUM+Z/FAC_J/FAC_J
      END;
      SERIES_ZS:=-1+2*A2*SUM/N
    END
    ELSE SERIES_ZS:=-1
  END
END;

FUNCTION A_LAMBDA:EXTENDED;
(* Calculation of magnetic contribution to Helmholtz energy.
   Inden's formalism requiring a1,m,a2,n: see me-freepascal-manual.doc: eqn. (11.5)
 *)
VAR PFAC,TC,BM,TAU,MULT,DUM,A1,M,A2,N:EXTENDED;
BEGIN
  A1:=0; M:=0; A2:=0; N:=0;
  (* PFAC = INDEN_P in Inden's model *)
  PFAC:=INDEN_P;
  IF PFAC>1E-11 THEN
  BEGIN
    (* Determine Tau=T/Tc and Tc is pressure dependent: magnetic moment in BM. *)
    TC:=0; BM:=0; TC_AND_BETA(TC,BM,A1,M,A2,N);
    (* In a mixture TC may be zero: no magnetic contribution *)
    MULT:=NUM_MAG_ATOMS;
    IF TC>1E-5 THEN
    BEGIN
      IF TEMPERATURE>1E-3 THEN
      BEGIN
        TAU:=TEMPERATURE/TC;
        (* Inden's model *)
        A_LAMBDA:=MULT*GAS_CONSTANT*TEMPERATURE*G_TAU(TAU,A1,M,A2,N)*LN(1+BM)
      END
      ELSE
      BEGIN
        (* Inden's model: Zero Kelvin magnetic Helmholtz contribution *)
        DUM:=2*A1*SERIES_SQUARE_M(TAU,M)/M-1+2*A2*SERIES_SQUARE_N(TAU,N)/N;
        A_LAMBDA:=MULT*DUM*GAS_CONSTANT*TC*LN(1+BM)
      END
    END
    ELSE
    BEGIN
      (* When tau>1e-5 we have that g(tau)=-1 and A=-nRTln(1+bm) *)
      A_LAMBDA:=-MULT*GAS_CONSTANT*TEMPERATURE*LN(1+BM)
    END
  END
  ELSE A_LAMBDA:=0
END;

FUNCTION G_LAMBDA:EXTENDED;
(* Calculation of magnetic contribution to Gibbs energy. *)
VAR PFAC,TC,BM,A_L,A1,M,A2,N:EXTENDED;
BEGIN
  A1:=0; M:=0; A2:=0; N:=0;
  PFAC:=INDEN_P;
  IF PFAC>1E-11 THEN
  BEGIN
    (* At zero Kelvin there is also a contribution *)
    TC_AND_BETA(TC,BM,A1,M,A2,N);
    (* In a mixture Tcurie can be zero: the magnetic contribution to A is
       -nRTln(1+bm). In the endmember bm=0, but in the mixture it may be somewhat
       larger, due to the correction in Bm in e.g. bohrmag.pas (external program)
     *)
    IF TC>1E-5 THEN
    BEGIN
      (* Magnetic correction factor is normally 1, but may deviate from 1 when
         magnetism is connected with crystal-field
       *)
      A_L:=A_LAMBDA*MAGNETIC_CORR_FACTOR;
      G_LAMBDA:=A_L
    END
    ELSE
    BEGIN
      (* At TC<1e-5 we have tau->infinity: we have adopted this in A_lambda and
         P_lambda where tc_and_beta() is calculated again
       *)
      G_LAMBDA:=A_LAMBDA*MAGNETIC_CORR_FACTOR
    END
  END
  ELSE G_LAMBDA:=0
END;

FUNCTION S_LAMBDA:EXTENDED;
(* Inden's formalism requiring a1,m,a2,n: see me-freepascal-manual.doc, eqn. (11.8) *)
VAR PFAC,TC,BM,TAU,MULT,A1,M,A2,N:EXTENDED;
BEGIN
  A1:=0; M:=0; A2:=0; N:=0;
  (* PFAC is only required in museum Calphad, not in Inden's model *)
  PFAC:=INDEN_P;
  IF PFAC>1E-11 THEN
  BEGIN
    (* At zero Kelvin the contribution to entropy is zero *)
    IF TEMPERATURE>1E-3 THEN
    BEGIN
      TC_AND_BETA(TC,BM,A1,M,A2,N);
      MULT:=NUM_MAG_ATOMS;
      (* In a mixture Tcurie can be zero: no magnetic contribution *)
      IF TC>1E-5 THEN
      BEGIN
        TAU:=TEMPERATURE/TC;
        (* Inden's model *)
        S_LAMBDA:=-GAS_CONSTANT*MULT*SERIES_ZS(TAU,A1,M,A2,N)*LN(1+BM)
      END
      ELSE
      BEGIN
        (* When Tc<1e-5 means that tau>1e5: in that case is series_zs=-1
           and S_lambda=nRln(1+bm)
         *)
        S_LAMBDA:=GAS_CONSTANT*MULT*LN(1+BM)
      END
    END
    ELSE S_LAMBDA:=0
  END
  ELSE S_LAMBDA:=0
END;

FUNCTION CV_LAMBDA:EXTENDED;
(* Isochoric heat capacity: see me-freepascal-manual.doc: eqn. (11.9).
   MULT: In MgO-SiO2-FeO, Fe2SiO4 is defined as (FeO)2/3(SiO2)1/3 and MULT=2/3.
 *)
VAR PFAC,TC,BM,Z1,TAU,MULT,A1,M,A2,N:EXTENDED;
BEGIN
  A1:=0; M:=0; A2:=0; N:=0;
  (* PFAC is only required in museum Calphad, not in Inden's model *)
  PFAC:=INDEN_P;
  IF PFAC>1E-11 THEN
  BEGIN
    (* At zero Kelvin the contribution to heat capacity is zero *)
    IF TEMPERATURE>1E-3 THEN
    BEGIN
      (* Determine Tau=T/Tc and Tc is pressure dependent: magnetic moment in
         BM. DTC, DBM are 1st volume derivatives of Tc and BM and are written
         e.g. as dTc/dV a constant T. D2TC, D2BM are the 2nd volume derivatives
       *)
      TC_AND_BETA(TC,BM,A1,M,A2,N);
      MULT:=NUM_MAG_ATOMS;
      (* In a mixture Tcurie can be zero: no magnetic contribution *)
      IF TC>1E-5 THEN
      BEGIN
        TAU:=TEMPERATURE/TC;
        (* Inden's model *)
        Z1:=-TAU*SERIES_ZC(TAU,A1,M,A2,N);
        CV_LAMBDA:=GAS_CONSTANT*MULT*Z1*LN(1+BM)
      END
      ELSE CV_LAMBDA:=0
    END
    ELSE CV_LAMBDA:=0
  END
  ELSE CV_LAMBDA:=0
END;

PROCEDURE GET_MAGNETIC_PROPERTIES(VAR A_MAG,U_MAG,S_MAG,CV_MAG:EXTENDED);
(* Calculate magnetic contributions to Helmholtz energy, energy, entropy and
   isochoric heat capacity
 *)
BEGIN
  A_MAG:=0; U_MAG:=0; S_MAG:=0; CV_MAG:=0;
  (* Magnetic correction factor MAGNETIC_CORR_FACTOR may be defined a number
     deviating from 1 when the magnetic expressions are used for the LAMBDA
     contribution in crystal-field
   *)
  IF MAGNETIC_YES>0 THEN
  BEGIN
    (* Helmholtz energy *)
    A_MAG:=A_LAMBDA*MAGNETIC_CORR_FACTOR;
    (* Entropy *)
    S_MAG:=S_LAMBDA*MAGNETIC_CORR_FACTOR;
    (* Energy *)
    U_MAG:=A_MAG+TEMPERATURE*S_MAG;
    (* Isochoric heat capacity *)
    CV_MAG:=CV_LAMBDA*MAGNETIC_CORR_FACTOR
  END
END;

END.
