UNIT maketable;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   This unit saves the output to a table
 *)

INTERFACE
USES CRT,sysutils,variables,readinput,magnetic,makevolume,isenhugo;


PROCEDURE STORE_2_TABLE(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,
                  DPDT,ALPHA,ZERO_PNT,SHEARMOD:EXTENDED);
PROCEDURE INITIALIZE_SCREEN;
PROCEDURE INITIALIZE_OUTPUTFILE(V0_ST_FIX:EXTENDED;FIX_YES:BOOLEAN);
PROCEDURE MAKE_TABLE;
IMPLEMENTATION

TYPE STR3=STRING[3];
VAR PRESSURE_UNIT:STR3;
    FACDIV,DENSITY,KS,VB,VP,VS:EXTENDED;
    

PROCEDURE GET_PRESSURE_UNIT(VAR FACDIV:EXTENDED;VAR PRESSURE_UNIT:STR3);
(* Determine the pressure unit *)
VAR MAX_PRESSURE:EXTENDED;
BEGIN
  IF P_START>P_END THEN MAX_PRESSURE:=P_START ELSE MAX_PRESSURE:=P_END;
  PRESSURE_UNIT:='Pa '; FACDIV:=1;
  IF MAX_PRESSURE>=1E+5 THEN
  BEGIN
    FACDIV:=1E3; PRESSURE_UNIT:='KPa'
  END;
  IF MAX_PRESSURE>=1E+6 THEN
  BEGIN
    FACDIV:=1E6; PRESSURE_UNIT:='MPa'
  END;
  IF MAX_PRESSURE>=1E+9 THEN
  BEGIN
    FACDIV:=1E9; PRESSURE_UNIT:='GPa'
  END;
  IF MAX_PRESSURE>=1E+12 THEN
  BEGIN
    FACDIV:=1E12; PRESSURE_UNIT:='TPa'
  END
END;

PROCEDURE INITIALIZE_SCREEN;
(* Sets table header on screen *)
VAR J:INTEGER;
BEGIN
  WINDOW(1,1,80,24); CLRSCR;
  (* Type of calculation *)
  GOTOXY(1,1);
  CASE CALCULATION_TYPE OF
    1:WRITE('Isobaric calculation');
    2:WRITE('Isothermal calculation');
    4:WRITE('Isentropic calculation');
    5:WRITE('Hugoniot calculations')
  END;
  (* Substance name *)
  GOTOXY(1,2); WRITE('Substance name: ',SUBSTANCE_NAME);
  (* Give start pressure/temperature to keep track when long calculations *)
  FACDIV:=1; GET_PRESSURE_UNIT(FACDIV,PRESSURE_UNIT);
  GOTOXY(1,3);
  CASE CALCULATION_TYPE OF
    1:WRITE('Pressure starts at: ',PRESSURE/FACDIV:10:4,' ',PRESSURE_UNIT);
    2:WRITE('Temperture starts at: ',TEMPERATURE:10:4,' Kelvin');
    4:WRITE('Constant entropy: ',S_ISENTROPE:10:4,' J/K/mol');
    5:WRITE('Sample temperature at 1 bar: ',T_HUGO_1BAR:10:4,' Kelvin')
  END;
  GOTOXY(1,4);
  WRITE('    T/K        P/',PRESSURE_UNIT);
  WRITE('    V/cm3/mol  d/gr/cm3   a*1E+4/K   K/GPa     H/J/mol');
  GOTOXY(1,5);
  FOR J:=1 TO 5 DO WRITE('-');
  WRITE(' << esc=break >> ');
  FOR J:=1 TO 57 DO WRITE('-');
  WINDOW(1,6,80,24); GOTOXY(1,1)
END;

PROCEDURE INITIALIZE_OUTPUTFILE(V0_ST_FIX:EXTENDED;FIX_YES:BOOLEAN);
(* Sets table header in the output file *)
VAR MONTH_NAME:STR14;
    YEAR,DAY:STR4;
    HOUR,MINUTE,SECOND:STR2;
    DIFFERENCE,RELATIVE_DIFFERENCE:EXTENDED;
    J:INTEGER;
BEGIN
  (* Name of the program that constructed the output file *)
  WRITELN(F_OUTPUT,'This file calculated by: me',ME_VERSION);
  (* Get path+name of input file: see unit READINPUT.PAS *)
  GET_PATH_INPUTFILE(NAME_INPUTFILE);
  WRITELN(F_OUTPUT,'Input file: ',NAME_INPUTFILE);
  (* Date-Time this file is constructed: in unit VARIABLES.PAS *)
  CURRENT_DATE_TIME(YEAR,MONTH_NAME,DAY,HOUR,MINUTE,SECOND);
  WRITE(F_OUTPUT,'Constructed on   : ',MONTH_NAME,' ',DAY,', ',YEAR);
  WRITELN(F_OUTPUT,' / ',HOUR,'hr:',MINUTE,'min:',SECOND,'sec');
  (* Type of calculation *)
  CASE CALCULATION_TYPE OF
    1:WRITELN(F_OUTPUT,'Isobaric calculation');
    2:WRITELN(F_OUTPUT,'Isothermal calculation');
    4:WRITELN(F_OUTPUT,'Isentropic calculation');
    5:WRITELN(F_OUTPUT,'Hugoniot calculation')
  END;
  (* Substance name *)
  WRITELN(F_OUTPUT,'Substance name: ',SUBSTANCE_NAME);
  (* Property that is kept constant *)
  FACDIV:=1; GET_PRESSURE_UNIT(FACDIV,PRESSURE_UNIT);
  CASE CALCULATION_TYPE OF
    1:WRITELN(F_OUTPUT,'Pressure starts at: ',PRESSURE/FACDIV:10:4,' ',PRESSURE_UNIT);
    2:WRITELN(F_OUTPUT,'Temperature starts at: ',TEMPERATURE:10:4,' Kelvin');
    4:WRITELN(F_OUTPUT,'Constant entropy: ',S_ISENTROPE:10:4,' J/K/mol');
    5:WRITELN(F_OUTPUT,'Sample temperature at 1 bar: ',T_HUGO_1BAR:10:4,' Kelvin')
  END;
  IF FIX_YES=TRUE THEN
  BEGIN
    (* You changed input parameters. FIX_STATIC_VOLUME() in unit MAKEVOLUME.PAS
       changed the static volume
     *)
    FOR J:=1 TO 80 DO WRITE(F_OUTPUT,'#');
    WRITELN(F_OUTPUT);
    WRITE(F_OUTPUT,'Model parameters were changed or preset in the input file. Therefore the total');
    WRITELN(F_OUTPUT);
    WRITELN(F_OUTPUT,'equation of state pressure, P_total(T=0,V0), does not match the external');
    WRITE(F_OUTPUT,'pressure at (0 K, 0 Pa).');
    WRITELN(F_OUTPUT);
    WRITE(F_OUTPUT,'''me'' repaired this by changing the static volume. The input parameters now give');
    WRITELN(F_OUTPUT);
    WRITE(F_OUTPUT,'P_total(T=0,V0) = 0 when the external pressure is zero.');
    WRITELN(F_OUTPUT); WRITELN(F_OUTPUT);
    WRITELN(F_OUTPUT,'Old static volume: ',V0_ST*1E+06:10:6,' cm3/mol');
    WRITELN(F_OUTPUT,'New static volume: ',V0_ST_FIX*1E+06:10:6,' cm3/mol');
    DIFFERENCE:=V0_ST_FIX-V0_ST; RELATIVE_DIFFERENCE:=100*DIFFERENCE/V0_ST;
    WRITE(F_OUTPUT,'Difference       : ',DIFFERENCE*1E+06:10:6,' cm3/mol');
    WRITELN(F_OUTPUT,'   [',RELATIVE_DIFFERENCE:10:6,'%]');
    FOR J:=1 TO 80 DO WRITE(F_OUTPUT,'#');
    WRITELN(F_OUTPUT)
  END;
  WRITE(F_OUTPUT,'    T/K        P/',PRESSURE_UNIT);
  WRITE(F_OUTPUT,'    V/cm3/mol  d/gr/cm3      a*1E+4/K     K/GPa     Cp/J/K/mol');
  WRITE(F_OUTPUT,' Cv/J/K/mol   Ks/GPa           H/J/mol     S/J/K/mol');
  WRITE(F_OUTPUT,'        G/J/mol    vb/km/s    vp/km/s    vs/km/s    Gshear/GPa');
  WRITELN(F_OUTPUT);
  FOR J:=1 TO 196 DO WRITE(F_OUTPUT,'-');
  WRITELN(F_OUTPUT)
END;

PROCEDURE GET_OTHER_PROPERTIES(VPT,CP,CV,BULKMOD,SHEARMOD:EXTENDED;VAR DENSITY,
                               KS,VB,VP,VS:EXTENDED);
(* Gets additional properties derivable from the output. VB=bulk sound velocity,
   VP=longitudinal sound velocity, VS=shear shound velocity, DENSITY=density,
   KS=adiabatic bulk modulus
 *)
VAR DUMMY:EXTENDED;
BEGIN
  (* MOLAR_MASS in gr/mol, VPT in m3/mol, BULKMOD, SHEARMOD in Pascal *)
  DENSITY:=MOLAR_MASS/VPT/1E6;
  IF ABS(CV)>0 THEN KS:=BULKMOD*CP/CV ELSE KS:=BULKMOD;
  DUMMY:=1000*KS*VPT/MOLAR_MASS;
  IF DUMMY>0 THEN VB:=SQRT(DUMMY) ELSE VB:=0;
  (* Make unit of velocity km/s *)
  VB:=VB/1000;
  DUMMY:=1000*SHEARMOD*VPT/MOLAR_MASS;
  IF DUMMY>0 THEN VS:=SQRT(DUMMY) ELSE VS:=0;
  (* Make unit of velocity km/s *)
  VS:=VS/1000;
  DUMMY:=VB*VB+4*VS*VS/3;
  (* Make unit of velocity km/s automatically *)
  IF DUMMY>0 THEN VP:=SQRT(DUMMY) ELSE VP:=0
END;

PROCEDURE VALUES_TO_SCREEN(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,
                  DPDT,ALPHA,ZERO_PNT,SHEARMOD,DENSITY,KS,VB,VP,VS:EXTENDED);
(* Write values to screen *)
VAR DUMMY:DOUBLE;
BEGIN
  FACDIV:=1;
  GET_PRESSURE_UNIT(FACDIV,PRESSURE_UNIT);
  IF TEMPERATURE<9999.9999 THEN WRITE(TEMPERATURE:10:4,' ')
  ELSE WRITE(TEMPERATURE:10,' ');
  DUMMY:=PRESSURE/FACDIV;
  IF DUMMY<99999.9999 THEN WRITE(DUMMY:10:4,' ')
  ELSE WRITE(DUMMY:10,' ');
  (* Volume in cm3/mol *)
  WRITE(VPT*1E+06:10:6,' ');
  WRITE(DENSITY:10:6,' ');
  (* Expansivity *)
  DUMMY:=ALPHA*1E+04;
  IF ABS(DUMMY)<999 THEN WRITE(DUMMY:10:6,' ') ELSE WRITE(DUMMY:10,' ');
  (* Isothermal bulk modulus *)
  DUMMY:=BULKMOD/1E9;
  IF DUMMY<9999.9999 THEN WRITE(DUMMY:10:4,' ')
  ELSE WRITE(DUMMY:10,' ');
  (* Enthalpy *)
  DUMMY:=ENTHALPY;
  IF ABS(DUMMY)<9.99999E+05 THEN WRITE(DUMMY:11:3)
  ELSE WRITE(DUMMY:11);
  WRITELN;
END;

PROCEDURE VALUES_TO_FILE(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,
                  DPDT,ALPHA,ZERO_PNT,SHEARMOD,DENSITY,KS,VB,VP,VS:EXTENDED);
(* Write values to file *)
VAR DUMMY:DOUBLE;
BEGIN
  FACDIV:=1;
  GET_PRESSURE_UNIT(FACDIV,PRESSURE_UNIT);
  IF TEMPERATURE<99999.9999 THEN WRITE(F_OUTPUT,TEMPERATURE:10:4,' ')
  ELSE WRITE(F_OUTPUT,TEMPERATURE:10,' ');
  DUMMY:=PRESSURE/FACDIV;
  IF DUMMY<9999.9999 THEN WRITE(F_OUTPUT,DUMMY:10:4,' ')
  ELSE WRITE(F_OUTPUT,DUMMY:10,' ');
  (* Volume in cm3/mol *)
  WRITE(F_OUTPUT,VPT*1E+06:10:6,' ');
  (* Density *)
  WRITE(F_OUTPUT,DENSITY:13:9,' ');
  (* Expansivity *)
  DUMMY:=ALPHA*1E+04;
  IF ABS(DUMMY)<999 THEN WRITE(F_OUTPUT,DUMMY:10:6,' ')
  ELSE WRITE(F_OUTPUT,DUMMY:10,' ');
  (* Bulk modulus unit is GPa *)
  DUMMY:=BULKMOD/1E9;
  IF DUMMY<9999.9999 THEN WRITE(F_OUTPUT,DUMMY:12:6,' ')
  ELSE WRITE(F_OUTPUT,DUMMY:12,' ');
  (* Isobaric heat capacity *)
  IF ABS(CP)<999 THEN WRITE(F_OUTPUT,CP:10:6,' ') ELSE WRITE(F_OUTPUT,CP:10,' ');
  (* Isochoric heat capacity *)
  WRITE(F_OUTPUT,CV:10:6,' ');
  (* Adiabatic bulk modulus unit is GPa *)
  DUMMY:=KS/1E9;
  IF DUMMY<9999.9999 THEN WRITE(F_OUTPUT,DUMMY:12:6,' ')
  ELSE WRITE(F_OUTPUT,DUMMY:12,' ');
  (* Enthalpy *)
  DUMMY:=ENTHALPY;
  IF ABS(DUMMY)<9.99999E+06 THEN WRITE(F_OUTPUT,DUMMY:15:6,' ')
  ELSE WRITE(F_OUTPUT,DUMMY:15,' ');
  (* Entropy *)
  WRITE(F_OUTPUT,ENTROPY:12:6,' ');
  (* Gibbs energy *)
  IF ABS(GIBBS)<9.99999E+06 THEN WRITE(F_OUTPUT,GIBBS:15:6,' ')
  ELSE WRITE(F_OUTPUT,GIBBS:15,' ');
  (* Bulk sound velocity *)
  WRITE(F_OUTPUT,VB:10:6,' ');
  (* Longitudinal sound velocity *)
  WRITE(F_OUTPUT,VP:10:6,' ');
  (* Shear sound velocity *)
  WRITE(F_OUTPUT,VS:10:6,' ');
  (* Shear modulus unit is GPa *)
  DUMMY:=SHEARMOD/1E9;
  IF DUMMY<9999.9999 THEN WRITE(F_OUTPUT,DUMMY:12:6,' ')
  ELSE WRITE(F_OUTPUT,DUMMY:12,' ');
  WRITELN(F_OUTPUT)
END;

PROCEDURE STORE_2_TABLE(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,
                  DPDT,ALPHA,ZERO_PNT,SHEARMOD:EXTENDED);
(* Store output to table *)
BEGIN
  GET_OTHER_PROPERTIES(VPT,CP,CV,BULKMOD,SHEARMOD,DENSITY,KS,VB,VP,VS);
  IF TABLE_SCREEN=1 THEN
  BEGIN
    VALUES_TO_SCREEN(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,
                  DPDT,ALPHA,ZERO_PNT,SHEARMOD,DENSITY,KS,VB,VP,VS);
  END;
  VALUES_TO_FILE(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,
                 DPDT,ALPHA,ZERO_PNT,SHEARMOD,DENSITY,KS,VB,VP,VS);
END;

PROCEDURE MAKE_TABLE;
(* Main part for calculating thermodynamic properties for output to table.
   CALCULATION_TYPE: 1=isothermal, 2=isobaric, 4=isentrope, 5=hugoniot
 *)
VAR VPT,GIBBS,HELMHOLTZ,ENERGY,ENTHALPY,ENTROPY,CP,CV,BULKMOD,DPDT,ALPHA,
    ZERO_PNT,SHEARMOD,V0_ST_FIX,U_FIX,V_FIX:EXTENDED;
    COUNT:INTEGER;
    FIX_YES,LAST_INNER_CALCULATION,LAST_OUTER_CALCULATION:BOOLEAN;
BEGIN
  (* Check if user changed model parameters. In that case the static volume must be
     changed to keep a stable lattice at 0 K, 0 Pa.
     Initialize volume that must be kept in memory for making extrapolations
     and set extrapolation True only when more than 2 calculation have been
     carried out. Extrapolation speeds-up calculations.
     For static volume correction no extrapolation required.
     V0 is the volume at P=T=0 and not the static volume
   *)
  FIX_YES:=FALSE; EXTRAPOLATE:=FALSE; VMEMORY:=V0; OLD_VMEMORY:=V0;
  (* FIX_STATIC_VOLUME is in unit MAKEVOLUME.PAS *)
  FIX_STATIC_VOLUME(V0_ST_FIX,FIX_YES);
  (* Determine constants A1, A2 of the McLaurin series expansion in case of magnetic
     contribution. CONVERT_MAGNETICS is in unit MAGNETIC.PAS
   *)
  CONVERT_MAGNETICS;
  (* Set start values for pressure/temperature
     CALCULATION_TYPE: 1=isobar, 2=isotherm
   *)
  CASE CALCULATION_TYPE OF
    1:BEGIN
        TEMPERATURE:=T_START-T_STEP; PRESSURE:=P_START
      END;
    2:BEGIN
        PRESSURE:=P_START-P_STEP; TEMPERATURE:=T_START
      END
  END;
  (* Main control part of the calculation *)
  LAST_INNER_CALCULATION:=FALSE; LAST_OUTER_CALCULATION:=FALSE; STOP_CALC_KEY:=FALSE;
  (* Set output file open for writing and write a header with properties.
     The required procedure is in unit READINPUT.PAS
   *)
  SET_WRITE_FILE(F_OUTPUT,NAME_OUTPUTFILE);
  (* If it is necessary to fix the static lattice volume, give a warning in 
     the output file. The old value for static volume is needed for that.
   *)
  INITIALIZE_OUTPUTFILE(V0_ST_FIX,FIX_YES);
  (* Now change the static volume, only if necessary *)
  IF FIX_YES=TRUE THEN V0_ST:=V0_ST_FIX;
  (* Initialize thermo properties for Isentrope or Hugoniot *)
  GIBBS:=0; HELMHOLTZ:=0; ENTHALPY:=0; ENERGY:=0; ENTROPY:=0; CP:=0; CV:=0;
  BULKMOD:=0; DPDT:=0; ALPHA:=0; ZERO_PNT:=0;
  (* In case of Hugoniot calculation, determine energy and volume at the initial
     condition of the sample T_HUGO_1BAR, 1 bar
   *)
  IF CALCULATION_TYPE=5 THEN
  BEGIN
    TEMPERATURE:=T_HUGO_1BAR; PRESSURE:=1E5; VPT:=V0;
    GET_THERMO_PROPERTIES(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,
                          BULKMOD,DPDT,ALPHA,ZERO_PNT);
    U_FIX:=ENERGY; V_FIX:=VPT;
    PRESSURE:=P_START-P_STEP
  END;
  IF CALCULATION_TYPE=4 THEN
  BEGIN
    PRESSURE:=P_START-P_STEP;
    (* Estimate adiabatic temperature *)
    TEMPERATURE:=300
  END;
  REPEAT
    (* Initialize screen output when required *)
    IF TABLE_SCREEN>0 THEN INITIALIZE_SCREEN;
    EXTRAPOLATE:=FALSE; LAST_INNER_CALCULATION:=FALSE;
    COUNT:=0; VMEMORY:=V0; OLD_VMEMORY:=V0;
    (* Inner calculation: increase temperature along isobars of increase pressure
       along isotherms, isentrope and hugoniot
     *)
    REPEAT
      CASE CALCULATION_TYPE OF
        1:BEGIN
            TEMPERATURE:=TEMPERATURE+T_STEP;
            IF (TEMPERATURE>=T_END) AND (T_STEP>0) THEN LAST_INNER_CALCULATION:=TRUE;
            IF (TEMPERATURE<=T_END) AND (T_STEP<0) THEN LAST_INNER_CALCULATION:=TRUE;
            IF ABS(T_STEP)<1E-12 THEN LAST_INNER_CALCULATION:=TRUE
          END;
    2,4,5:BEGIN
            PRESSURE:=PRESSURE+P_STEP;
            IF (PRESSURE>=P_END) AND (P_STEP>0) THEN LAST_INNER_CALCULATION:=TRUE;
            IF (PRESSURE<=P_END) AND (P_STEP<0) THEN LAST_INNER_CALCULATION:=TRUE;
            IF ABS(P_STEP)<1E-12 THEN LAST_INNER_CALCULATION:=TRUE
          END;
      END;
      COUNT:=COUNT+1;
      IF COUNT>2 THEN
      BEGIN
        (* If count>2 extrapolation of volume and step is possible in CALC_VPT()
           of unit MAKE_VOLUME.PAS for faster calculations
         *)
        COUNT:=3; EXTRAPOLATE:=TRUE
      END;
      (* In case values are written to screen, check if a user break. The required
         procedure is in unit READINPUT.PAS
       *)
      IF TABLE_SCREEN=1 THEN GET_STOP_CALC_KEY;
      (* In case of isentrope or hugoniot determine temperature and volume (VPT) at
         pressure. ISENHUGO_CALC() is in unit ISENHUGO.PAS
       *)
      IF (CALCULATION_TYPE>3) AND (CALCULATION_TYPE<6) THEN
        ISENHUGO_CALC(CALCULATION_TYPE,U_FIX,S_ISENTROPE,V_FIX,VPT);
      (* Get thermodynamic properties in unit MAKEVOLUME.PAS *)
      GET_THERMO_PROPERTIES(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,
                            BULKMOD,DPDT,ALPHA,ZERO_PNT);
      (* Calculate shear modulus, in unit MAKEVOLUME.PAS *)
      SHEARMOD:=SHEAR_MODULUS(VPT);
      (* Tables are stored in this unit *)
      STORE_2_TABLE(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,
                    DPDT,ALPHA,ZERO_PNT,SHEARMOD);
    UNTIL
      (LAST_INNER_CALCULATION=TRUE) OR (STOP_CALC_KEY=TRUE);
    IF STOP_CALC_KEY=FALSE THEN
    BEGIN
      (* When isobars increase the pressure. When isotherms increase temperature *)
      CASE CALCULATION_TYPE OF
        1:BEGIN
            PRESSURE:=PRESSURE+P_STEP; TEMPERATURE:=T_START-T_STEP;
            IF (PRESSURE>P_END) AND (P_STEP>0) THEN LAST_OUTER_CALCULATION:=TRUE;
            IF (PRESSURE<P_END) AND (P_STEP<0) THEN LAST_OUTER_CALCULATION:=TRUE;
            IF ABS(P_STEP)<1E-12 THEN LAST_OUTER_CALCULATION:=TRUE
          END;
        2:BEGIN
            TEMPERATURE:=TEMPERATURE+T_STEP; PRESSURE:=P_START-P_STEP;
            IF (TEMPERATURE>T_END) AND (T_STEP>0) THEN LAST_OUTER_CALCULATION:=TRUE;
            IF (TEMPERATURE<T_END) AND (T_STEP<0) THEN LAST_OUTER_CALCULATION:=TRUE;
            IF ABS(T_STEP)<1E-12 THEN LAST_OUTER_CALCULATION:=TRUE
          END;
      4,5:BEGIN
            LAST_OUTER_CALCULATION:=TRUE
          END
      END;
    END
  UNTIL
    (LAST_OUTER_CALCULATION=TRUE) OR (STOP_CALC_KEY=TRUE);
  IF TABLE_SCREEN>0 THEN
  BEGIN
    IF STOP_CALC_KEY=FALSE THEN WRITELN('Normal termination...')
    ELSE WRITELN('User break...')
  END;
  (* Close output file *)
  CLOSE(F_OUTPUT)
END;

END.
