PROGRAM medos;
USES crt,sysutils;

(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Belongs to me.pas version 8, January 31st 2021, manual chapter 15.
   A VDoS has been made by reading-off the points from an experimental or ab initio
   VDoS plot. Because the VDoS often has arbitrary Y-axis units, the surface underneath
   the plot id scaled to 3.n.NA with NA Avogadro's number.
   The points in the vdos file should have been sorted to climbing wave number.
   Next the experimental VDoS is sliced in a number of optic continua, say N. The frequency
   range of one OC (optic continuum in Kieffer's, 1979 model) is nu_max/N. We define
   delta=nu_max/2/N as the half-width of all optic continua.
   The Vdos starts at 0,0 (is automatically checked and inserted if needed).Next a point
   x[1] is taken. The first OC ranges from x[1]-delta to
   x[1]+delta. Point x[1] is therefore in the middle of OC1. Because all OC's have the same
   width, a point x[j]=x[j-1]+2*delta=x[1]+(j-1)*2*delta. Inside one OC the VDoS values
   are averaged in 20 points to obtain a smoother curve. The function values g(x[j])
   are obtained via linear interpolation between the experimental points.
   The surface of OCj is 2*delta*g(x[j]) and the total surface under the VDoS is
   Sum(2*delta*g(x[j]), where the sum extends from 1 to N. The fraction of
   oscillators in OCj is g(x[j])/Sum(g(x[j]).
   The VDoS starts with x[1]=delta.
 *)

CONST ELEMENTARY_CHARGE=1.602176565e-19;      (* Elementary charge, Coulomb *)
      GAS_CONST=8.3144621;                    (* Gas constant *)
      PLANCK=6.62606957e-34;                  (* Planck's constant *)
      BOLTZMANN=1.3806488e-23;                (* Boltzmann's constant *)
      LIGHT_SPEED=2.99792458e+08;             (* Speed of light in vacuum *)
      AVOGADRO=6.02214129e+23;                (* Avogadro's constant *)
TYPE RY_MODE=ARRAY[1..1000] OF EXTENDED;
     RY_EXP=ARRAY[1..4000] OF EXTENDED;
VAR F_INPUT:TEXT;            
                             (* F_INPUT: file type for input file *)
    LINE,NAME_INPUTFILE,NAME_VDOSFILE,NAME_MEFILE:ANSISTRING;
                             (* LINE: string read from a file
                                NAME_INPUTFILE: Name of the input file for 'mevdos'
                                NAME_VDOSFILE: Name of file with experimental VDoS points,
                                NAME_MEFILE: Name of file that serves as input for 'me'
                              *)
    LENGTHLINE,N_EINSTEIN,N_OC,MAX_POINTS,VDOS_UNIT:INTEGER; 
                             (* LENGTHLINE: length of a line read from a file
                                N_EINSTEIN: Desired number of Einstein temperatures in VDoS
                                N_OC: Number of optic continua in VDoS
                                MAX_POINTS: number of experimental datapoints VDoS
                                VDOS_UNIT: 1=cm^-1, 2=THz, 3=meV
                              *)
    X_EXP,G_EXP:RY_EXP;      (* Experimental frequency, VDoS value *)
    X_CAL,G_CAL,FRAC,THETA:RY_MODE;
                             (* Frequency of the middle of a mode, its height, selected
                                gruneisen, mode_q parameter and fraction of oscillators,
                                and Einstein temperatures
                              *)
    DELTA,GAMMA0,GAMMA_INF,MODE_ANH,MODE_ANH_M,MODE_Q0:EXTENDED;
                             (* GAMMA0    : Preset value for Gruneisen parameter
                                DELTA     : Half-width of an optic continuum
                                GAMMA_INF : Preset Gruneisen parameter at infinite pressure
                                MODE_ANH  : Preset Intrinsic mode anharmonicity
                                MODE_ANH_M: Preset Intrinsic mode anharmonicity, volume dependent
                                MODE_Q0   : Preset mode-q parameter
                              *)



PROCEDURE SET_READ_FILE(VAR FIL:TEXT;FILENAME:ANSISTRING);
(* Assign a name of file to a file and set it open for reading *)
BEGIN
  ASSIGN(FIL,FILENAME);RESET(FIL)
END;

PROCEDURE SET_WRITE_FILE(VAR FIL:TEXT;FILENAME:ANSISTRING);
(* Assign a name of file to a file and set it open for writing *)
BEGIN
  ASSIGN(FIL,FILENAME);REWRITE(FIL)
END;

FUNCTION EXIST_FILE(TESTNAME:ANSISTRING):BOOLEAN;
(* Check if a file exists *)
VAR FF:TEXT;
BEGIN
  ASSIGN(FF,TESTNAME);
  {$I-}  RESET(FF);  {$I+}
  IF IORESULT=0 THEN
  BEGIN
    EXIST_FILE:=TRUE;CLOSE(FF)
  END
  ELSE EXIST_FILE:=FALSE
END;

PROCEDURE CHECK_PATH(NAME:ANSISTRING;VAR INPUT_OKAY:BOOLEAN);
(* Checks if path of the file with name NAME exists. If that is the case
   set INPUT_OKAY=TRUE otherwise set it FALSE.
 *)
VAR FIL:TEXT;
    NAME_DUMMY,PATH:ANSISTRING;
BEGIN
  (* Extract the path of the file *)
  PATH:=EXTRACTFILEPATH(NAME);
  (* Make a test name different from name because the rewrite command below will
     otherwise overwrite the existing file and empty it
   *)
  NAME_DUMMY:=PATH+'mj241019.59';
  ASSIGN(FIL,NAME_DUMMY);
  {$I-} REWRITE(FIL); {$I+}
  IF IORESULT=0 THEN
  BEGIN
    INPUT_OKAY:=TRUE; CLOSE(FIL); ERASE(FIL)
  END
  ELSE INPUT_OKAY:=FALSE
END;

FUNCTION CALC_LENGTH(S:ANSISTRING):INTEGER;
(* Determining the correct length of a string S read from a file.
   When the ordinal of s[length(s)] equals 13 we have length(s)-1
   characters on the line. If not we have length(s) characters
 *)
VAR L,M:INTEGER;V:CHAR;
BEGIN
  L:=LENGTH(S);
  IF L>0 THEN
  BEGIN
    V:=S[L]; M:=ORD(V);
    IF (M=10) OR (M=13) THEN CALC_LENGTH:=L-1 
    ELSE CALC_LENGTH:=L
  END
  ELSE CALC_LENGTH:=0
END;

PROCEDURE DETECTBLANK(INP_S:ANSISTRING;VAR STRT:INTEGER);
(* Finds a blank in the string INP_S starting at character at position STRT.
 *)
VAR LENG_INP_S:INTEGER;STOP_IT:BOOLEAN;
BEGIN
  STRT:=STRT-1;LENG_INP_S:=CALC_LENGTH(INP_S); STOP_IT:=FALSE;
  REPEAT
    STRT:=STRT+1;
    IF STRT<=LENG_INP_S THEN
    BEGIN
      IF (INP_S[STRT]=' ') OR (ORD(INP_S[STRT])=0) THEN STOP_IT:=TRUE
    END
  UNTIL
    (STOP_IT=TRUE) OR (STRT>=LENG_INP_S)
END;

PROCEDURE DETECTNOTBLANK(INP_S:ANSISTRING;VAR STRT:INTEGER);
(* Finds a character not equal to blank in a string INP_S starting at
   character at position STRT of INP_S.
 *)
VAR LENG_INP_S:INTEGER;STOP_IT:BOOLEAN;
BEGIN
  STRT:=STRT-1;LENG_INP_S:=CALC_LENGTH(INP_S); STOP_IT:=FALSE;
  REPEAT
    STRT:=STRT+1;
    IF STRT<=LENG_INP_S THEN
    BEGIN
      IF (INP_S[STRT]<>' ') AND (ORD(INP_S[STRT])<>0) THEN STOP_IT:=TRUE
    END
  UNTIL
    (STOP_IT=TRUE) OR (STRT>=LENG_INP_S)
END;

PROCEDURE READLINE(VAR FIL:TEXT;VAR LINE:ANSISTRING;VAR NUM_LINE:INTEGER);
(* Returns the next line not preceding with a comment marker '#' *)
VAR J:INTEGER;
    STOP_IT:BOOLEAN;
BEGIN
  STOP_IT:=FALSE;
  REPEAT
    READLN(FIL,LINE); NUM_LINE:=NUM_LINE+1; LENGTHLINE:=CALC_LENGTH(LINE);
    IF LENGTHLINE>0 THEN
    BEGIN
      J:=1; DETECTNOTBLANK(LINE,J);
      IF LINE[J]<>'#' THEN STOP_IT:=TRUE
    END
  UNTIL
    (EOF(FIL)) OR (STOP_IT=TRUE)
END;

PROCEDURE GET_STRING(VAR STRIN:ANSISTRING);
(* Gets string at the beginning of a line, e.g name of the substance *)
VAR J:INTEGER;
BEGIN
  J:=1; DETECTNOTBLANK(LINE,J); STRIN:='';
  REPEAT
    IF (LINE[J]<>' ') AND (J<=LENGTHLINE) THEN STRIN:=STRIN+LINE[J];
    J:=J+1;
  UNTIL
    (LINE[J]=' ') OR (J>LENGTHLINE)
END;

PROCEDURE WARN_WRONG_INPUT(NUM_LINE:INTEGER;PATH_OKAY,VDOS_PRESENT:BOOLEAN);
(* Warning for wrong input *)
BEGIN
  WRITELN; WRITELN;
  WRITELN('--------------------------------------------------------------');
  WRITELN('Input file: ',NAME_INPUTFILE);
  WRITELN('Error in line number: ',NUM_LINE);
  IF PATH_OKAY=FALSE THEN
  BEGIN
    WRITELN('Path of output file is wrong...')
  END
  ELSE
  BEGIN
    IF VDOS_PRESENT=FALSE THEN
    BEGIN
      WRITELN('Cannot find file with VDoS points...')
    END
    ELSE
    BEGIN
      WRITELN('Number of Einstein temperatures should be integer, or');
      WRITELN('Flag to set VDoS unit should be integer...')
    END
  END;
  WRITELN('Program terminates...');
  WRITELN('--------------------------------------------------------------');
  WRITELN; WRITELN
END;

PROCEDURE READ_INPUT_FILE(VAR INPUT_OKAY:BOOLEAN);
(* Reads the input file *)
VAR STRIN:ANSISTRING;
    CODE,NUM_LINE:INTEGER;
    PATH_OKAY,VDOS_PRESENT:BOOLEAN;
BEGIN
  INPUT_OKAY:=TRUE; PATH_OKAY:=TRUE; VDOS_PRESENT:=TRUE; NUM_LINE:=0;
  SET_READ_FILE(F_INPUT,NAME_INPUTFILE);
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Desired number of Einstein temperatures in VDoS *)
    READLINE(F_INPUT,LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,N_EINSTEIN,CODE);
    (* Number of continua equals number of desired Einstein temperatures *)
    N_OC:=N_EINSTEIN;
    IF (CODE<>0) OR (N_OC<1) THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Output file, serving as input file for 'me' *)
    READLINE(F_INPUT,LINE,NUM_LINE);
    STRIN:=''; GET_STRING(NAME_MEFILE);
    CHECK_PATH(NAME_MEFILE,PATH_OKAY);
    IF PATH_OKAY=FALSE THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* VDoS unit of experimental/abinitio VDoS *)
    READLINE(F_INPUT,LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,VDOS_UNIT,CODE);
    IF CODE<>0 THEN INPUT_OKAY:=FALSE;
    IF (VDOS_UNIT<1) OR (VDOS_UNIT>3) THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* VDoS unit of experimental/abinitio VDoS *)
    READLINE(F_INPUT,LINE,NUM_LINE);
    STRIN:=''; GET_STRING(NAME_VDOSFILE);
    VDOS_PRESENT:=EXIST_FILE(NAME_VDOSFILE);
    IF VDOS_PRESENT=FALSE THEN INPUT_OKAY:=FALSE
  END;
  CLOSE(F_INPUT);
  IF INPUT_OKAY=FALSE THEN WARN_WRONG_INPUT(NUM_LINE,PATH_OKAY,VDOS_PRESENT)
END;

PROCEDURE GET_PARAMETER_VALUE(VAR STRT:INTEGER;VAR X:EXTENDED;VAR INPUT_OKAY:BOOLEAN);
(* Gets value from a line starting at strt-st character *)
VAR STRIN:ANSISTRING;
    J,CODE:INTEGER;
BEGIN
  J:=STRT; DETECTNOTBLANK(LINE,J); STRIN:='';
  REPEAT
    IF (LINE[J]<>' ') AND (J<=LENGTHLINE) THEN STRIN:=STRIN+LINE[J];
    J:=J+1;
  UNTIL
    (LINE[J]=' ') OR (J>LENGTHLINE);
  VAL(STRIN,X,CODE);
  IF CODE<>0 THEN INPUT_OKAY:=FALSE;
  STRT:=J
END;

PROCEDURE GET_NEAREST(X:EXTENDED;VAR J_LOW,J_UP:INTEGER);
(* X is a point in an OC. J_low is the number of the point in the experimental
   VDoS just below X and J_UP the number of the point just higher as X
 *)
VAR J:INTEGER;
    STOP_IT:BOOLEAN;
BEGIN
  J:=0; STOP_IT:=FALSE;
  REPEAT
    J:=J+1;
    IF X_EXP[J]>X THEN STOP_IT:=TRUE
  UNTIL
    (J>=MAX_POINTS) OR (STOP_IT=TRUE);
  J_UP:=J; J_LOW:=J-1;
  IF J_LOW=0 THEN
  BEGIN
    (* In unforseen case when the experimental points start at values above X *)
    J_LOW:=1; J_UP:=2
  END
END;

PROCEDURE MAKE_OPTIC;
(* Make arrays containing the midpoint frequency of optic continua and their top (height).
   Also make the fraction of oscillators in each optic continuum
 *)
VAR DELTA_B,G,X0,SUM,SUM_G,X:EXTENDED;
    I,J,J_LOW,J_UP,SUM_N:INTEGER;
BEGIN
  (* First x[1] in VDoS starts at delta and the 1st continuum goes from x[1]-delta to
     x[1]+delta and the total width is 2*delta
   *)
  X0:=-DELTA; J_LOW:=1; J_UP:=2;
  FOR I:=1 TO N_OC DO
  BEGIN
    X_CAL[I]:=X0+2*DELTA; X0:=X_CAL[I];
    (* X0 is now the midpoint of the continuum. Determine the top of the VDoS by
       averaging the VDoS heights between [X0-delta, X0+delta] with 20 points,
       spacing 2*delta/20. This is a smooting operation. The dataset of the
       experimental VDoS is in (X_EXP[1..max_points], G_EXP[1..max_points]). For
       each of the 20 points we must find the nearest neighbours in the EXP dataset.
     *)
    DELTA_B:=DELTA/10; X:=X0-DELTA; SUM_G:=0; SUM_N:=0;
    FOR J:=1 TO 20 DO
    BEGIN
      GET_NEAREST(X,J_LOW,J_UP);
      (* Get the height by interpolation between the experimental function values of the
         VDoS
       *)
      G:=(G_EXP[J_UP]-G_EXP[J_LOW])*(X-X_EXP[J_UP]);
      IF ABS(X_EXP[J_LOW]-X_EXP[J_UP])>1E-10 THEN
      BEGIN
        SUM_G:=SUM_G+G/(X_EXP[J_UP]-X_EXP[J_LOW])+G_EXP[J_UP];
        SUM_N:=SUM_N+1
      END;
      X:=X+DELTA_B
    END;
    (* Make average and we have a continuum defined by (X_CAL[I], G_CAL[I]) and
       width 2*delta between [X_CAL[I]-delta,X_CAL[I]+delta]
     *)
    G_CAL[I]:=SUM_G/SUM_N
  END;
  (* Avoid zero frequency: if x_cal[1]-delta=0 make it 0.0001 *)
  IF ABS(X_CAL[1]-DELTA)<1e-05 THEN X_CAL[1]:=0.0001+DELTA;
  (* Make the fractions of oscillators in each optic continuum *)
  SUM:=0;
  FOR I:=1 TO N_OC DO SUM:=SUM+G_CAL[I];
  FOR I:=1 TO N_OC DO FRAC[I]:=G_CAL[I]/SUM
END;

PROCEDURE SAVE_STATIC_PARAMETERS(VAR F_OUT:TEXT);
(* Write static part of 'me' input file with dummy values *)
BEGIN
  WRITELN(F_OUT,'# ===================================================================');
  WRITELN(F_OUT,'# Note that values have been preset to those for MgO !!');
  WRITELN(F_OUT,'# Only Theta and Fractions were determined with program ''mevdos''...');
  WRITELN(F_OUT,'# ===================================================================');
  WRITELN(F_OUT,'#');
  WRITELN(F_OUT,'MgO (Preset...)            (* Name of the substance *)');
  WRITELN(F_OUT,'40.304                     (* Mass of the substance in gr/mol *)');
  WRITELN(F_OUT,' 2.00                      (* Number of atoms in 1 molecular formula unit *)');
  WRITELN(F_OUT,'-6.21093534291483E+005     (* Static lattice energy in J/mol *)');
  WRITELN(F_OUT,' 1.12027710146365E-005     (* Volume/m3/mol at zero K and zero Pa *)');
  WRITELN(F_OUT,' 1.10736889124136E-005     (* Volume/m3/mol of the static lattice at zero Pa *)');
  WRITELN(F_OUT,' 2                         (* 2=Birch-Murnaghan, 1=Vinet EoS of the static lattice *)');
  WRITELN(F_OUT,' 3                         (* Order of Birch-Murnaghan EoS of the static lattice *)');
  WRITELN(F_OUT,' 1.71365000000000E+011     (* Static lattice bulk modulus/Pa at zero Pa *)');
  WRITELN(F_OUT,' 4.18588900000000E+000     (* Pressure derivative of the static lattice bulk modulus at zero Pa *)');
  WRITELN(F_OUT,' 1.39801200000000E+011     (* Static lattice shear modulus/Pa at zero Pa *)');
  WRITELN(F_OUT,' 2.27420200000000E+000     (* Pressure derivative of the static lattice shear modulus at zero Pa *)');
  WRITELN(F_OUT,' 2.06860800000000E+000     (* Vibrational parameter of the shear modulus *)');
  WRITELN(F_OUT,' 0                         (* 0=No Landau contribution, 1=2nd order, 2=Tricritical, 3=1st order *)');
  WRITELN(F_OUT,' 0                         (* 0=No magnetic contribution, 1=magnetic contribution *)');
  WRITELN(F_OUT,' 0                         (* No electronic contribution *)');
  WRITELN(F_OUT,' 0                         (* 0=No cation disorder, 1=cation disorder or O''Neill and Navrotsky (1983,1984) *)');
  WRITELN(F_OUT,' 0                         (* 0=No monovacancy defects, 1=monovacancy defects: Dorogokupets & Oganov 2007 Phys Rev B 75:024115 *)');
END;

PROCEDURE MAKE_ME_SETUP;
(* Saves output in 'me' input file format *)
VAR F_OUT:TEXT;
    SUM,UNIT_CAL_X:EXTENDED;
    I:INTEGER;
BEGIN
  SET_WRITE_FILE(F_OUT,NAME_MEFILE);
  SAVE_STATIC_PARAMETERS(F_OUT);
  CASE VDOS_UNIT OF
    1:BEGIN
        (* Convert wavenumbers/cm-1 to Einstein temperatures *)
        UNIT_CAL_X:=100*PLANCK*LIGHT_SPEED/BOLTZMANN
      END;
    2:BEGIN
        (* Convert frequencies/THz to Einstein temperatures *)
        UNIT_CAL_X:=1E+12*PLANCK/BOLTZMANN
      END;
    3:BEGIN
        (* Convert energies/meV to Einstein temperatures *)
        UNIT_CAL_X:=1E-03*ELEMENTARY_CHARGE/BOLTZMANN
      END
  END;
  FOR I:=1 TO N_OC DO THETA[I]:=X_CAL[I]*UNIT_CAL_X;
  (* Write header above vibrational properties *)
  WRITELN(F_OUT,N_OC,'                         (* Number of Einstein modes *)');
  WRITE(F_OUT,' 2');
  WRITE(F_OUT,'                         (* 1=Gruneisen expression by Al''tshuler et al, 1987');
  WRITELN(F_OUT,' and 2=that by Stixrude & Lithgow-Bertelloni, 2005 *)');
  WRITELN(F_OUT,'#  j   Theta/K     Fraction          Gamma      Mode_m(q)   Gamma_inf  a0             z');
  WRITE(F_OUT,'# ');
  FOR I:=1 TO 93 DO WRITE(F_OUT,'-');
  WRITELN(F_OUT);
  (* Write properties in the table *)
  GAMMA0:=1.0000; MODE_Q0:=0.1; GAMMA_INF:=0; MODE_ANH:=1.0e-11;
  MODE_ANH_M:=0.0;
  FOR I:=1 TO N_OC DO
  BEGIN
    WRITE(F_OUT,I:4,' ');
    WRITE(F_OUT,THETA[I]:12:6,' ',FRAC[I]:17,' ',GAMMA0:10:7,' ',MODE_Q0:10:7,' ');
    WRITELN(F_OUT,' ',GAMMA_INF:10:7,' ',MODE_ANH:14,' ',MODE_ANH_M:10:7)
  END;
  (* Check the sum of all fractions to be one *)
  SUM:=0;
  FOR I:=1 TO N_OC DO SUM:=SUM+FRAC[I];
  WRITELN(F_OUT,'#');
  WRITELN(F_OUT,'# Check sum of all fractions: ',SUM:17);
  FOR I:=1 TO 95 DO WRITE(F_OUT,'#');
  WRITELN(F_OUT);
  CLOSE(F_OUT)
END;

PROCEDURE MAKE_SETUP;
(* Make output file serving as input for 'me'. *)
VAR F_VDOS,F_OUT:TEXT;
    X:EXTENDED;
    INPUT_OKAY:BOOLEAN;
    J,NUM,NUM_LINE:INTEGER;
BEGIN
  INPUT_OKAY:=TRUE; NUM_LINE:=0;
  (* First point in VDOS must be (0,0) *)
  NUM:=1; X_EXP[1]:=0; G_EXP[1]:=0;
  (* Input file with experimental VDOS *)
  SET_READ_FILE(F_VDOS,NAME_VDOSFILE);
  (* Read in all experimental points of the VDOS and store in arrays. *)
  WHILE (NOT EOF(F_VDOS)) AND (INPUT_OKAY=TRUE) DO
  BEGIN
    READLINE(F_VDOS,LINE,NUM_LINE);
    IF LENGTHLINE>0 THEN
    BEGIN
      (* When the last line is a comment line EOF(F_INPUT)=True it has no values *)
      J:=1; DETECTNOTBLANK(LINE,J);
      IF LINE[J]<>'#' THEN
      BEGIN
        (* X_EXP *)
        J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY);
        NUM:=NUM+1; MAX_POINTS:=NUM; X_EXP[NUM]:=X;
        IF X<0 THEN INPUT_OKAY:=FALSE;
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          (* G_EXP on the same line *)
          GET_PARAMETER_VALUE(J,X,INPUT_OKAY); G_EXP[NUM]:=X;
          IF X<0 THEN INPUT_OKAY:=FALSE
        END
      END
    END;
    IF NUM>1 THEN
    BEGIN
      IF X_EXP[NUM]<X_EXP[NUM-1] THEN INPUT_OKAY:=FALSE
    END
  END;
  CLOSE(F_VDOS);
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Half-width of an optic continuum *)
    DELTA:=X_EXP[MAX_POINTS]/N_OC/2;
    (* Make array of mid-points of all optic continua and their heights *)
    MAKE_OPTIC;
    (* Optic continua now defined by mid-point frequency and top (height) as
       X_Cal[1..N_Oc], G_cal[1..N_oc]: Now make 'me' input file
     *)
    SET_WRITE_FILE(F_OUT,NAME_MEFILE);
    MAKE_ME_SETUP;
    WRITELN;
    WRITELN('-------------------------------------------------------------------');
    WRITELN('Input file       : ',NAME_INPUTFILE);
    WRITELN('VDoS file        : ',NAME_VDOSFILE);
    WRITELN('Created ''me'' file: ',NAME_MEFILE);
    WRITELN('Only Einstein temperatures and fractions were determined from VDoS.');
    WRITELN('(See header of ''me'' file)');
    WRITELN('-------------------------------------------------------------------');
    WRITELN
  END
  ELSE
  BEGIN
    WRITELN;
    WRITELN('-------------------------------------------------------------------');
    WRITELN('File: ',NAME_VDOSFILE);
    WRITELN('Error in line number: ',NUM_LINE);
    WRITELN('Frequencies not in increasing order...');
    WRITELN('-------------------------------------------------------------------');
    WRITELN
  END
END;

PROCEDURE MAIN_VDOS;
(* Main part of making a setup file for 'me'. The input file looks like:
   60                        [* Desired number of Einstein temperatures *]
   /home/michel/mgo.mef      [* Output file serving as input file for 'me' *]
   1                         [* VDoS unit: 1=cm^-1, 2=THz, 3=MeV *]
   /home/michel/vdosmgo.exp  [* file containing VDoS (X,Y) points *]
 *)
VAR INPUT_OKAY,IT_EXISTS:BOOLEAN;
BEGIN
  IF PARAMCOUNT=1 THEN
  BEGIN
    (* Check if the input file is present containing VDoS unit and the filename
       containing the experimental/abinitio VDoS points
     *)
    NAME_INPUTFILE:=PARAMSTR(1);
    IT_EXISTS:=FALSE; IT_EXISTS:=EXIST_FILE(NAME_INPUTFILE);
    IF IT_EXISTS=TRUE THEN
    BEGIN
      READ_INPUT_FILE(INPUT_OKAY);
      IF INPUT_OKAY=TRUE THEN MAKE_SETUP
    END
    ELSE
    BEGIN
      (* Input file does not exist: terminate program *)
      WRITELN; WRITELN;
      WRITELN('----------------------------');
      WRITELN('Input file does not exist...');
      WRITELN('----------------------------');
      WRITELN; WRITELN
    END
  END
  ELSE
  BEGIN
    (* If no input file as argument is defined then PARAMCOUNT=0 *)
    WRITELN; WRITELN;
    WRITELN('-------------------------------------------------------------');
    WRITELN('Use this program in combination with an input file by typing:');
    WRITELN('./mevdos inputfile.vdos');
    WRITELN('next press enter');
    WRITELN('-------------------------------------------------------------');
    WRITELN; WRITELN
  END
END;

BEGIN
  MAIN_VDOS
END.
