UNIT variables;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   - Defining global constants, arrays and variables needed for the multiple
     Einstein method.
   - Common procedures and functions in other units.
 *)

INTERFACE
USES CRT,sysutils;

CONST NUMBER_OF_FREQUENCIES=200;              (* Maximum number of Einstein frequencies *)
      LIGHT_SPEED=2.99792458E+08;             (* Speed of light in vaccuum, m/s *)
      GAS_CONSTANT=8.3144621;                 (* Gas constant, J/K/mol *)
      BOLTZMANN=1.3806488E-23;                (* Boltzmann's constant, J/K *)
      AVOGADRO=6.02214129E+23;                (* Avogadro's constant, /mol *)
      PLANCK=6.62606957E-34;                  (* Planck's constant, Js *)
      MASS_ELECTRON=9.10938291E-31;           (* Mass of electron, kg *)
      ELEMENTARY_CHARGE=1.602176565e-19;      (* Elementary charge, Cb *)
      MAX_ARG_EXP_CONST=11200;                (* Maximum value for the argument of function
                                                 exp(): +11200. This value is assigned to
                                                 extended real max_arg_exp, because
                                                 exp(max_arg_exp_const) gives a runtime error
                                                 whereas exp(max_arg_exp) does not
                                               *)
      RESTRICT_EXP_X=1200;                    (* Restricts the evaluation of exp(x) to a value
                                                 of 1e+restrict_exp_x=1e+1200. That is
                                                 necessary because for Cv a power of
                                                 (exp(x)-1)^4 is taken for anharmonicity.
                                                 That may give 1e+4800, whereas 1e+4950 is the
                                                 maximum value for an extended real
                                               *)
      CRIT_ARG_EXP=1E-9;                      (* Minimum value of X=h*nu/kT to be evaluated by
                                                 function exp(X): 1E-09
                                               *)
      MAX_EOS_ORDER=8;                        (* Maximum order Equation of state *)
      NUMBER_OF_MCLAURIN_TERMS=50;            (* Magnetic: number of terms in the McLaurin
                                                 series of Cv derived from Inden's expression
                                               *)
      MAX_CRYSTFIELD_LEVELS=100;              (* Maximum number of energy levels in the electronic
                                                 crystal field description: this is the sum
                                                 of the number of energy levels of the M1 and
                                                 the M2 site
                                               *)
      MAX_VDOS_POINTS=10000;                  (* Max number of points in experimental/abinitio
                                                 VDoS
                                               *)


TYPE BULKMOD_DERIVATIVES=ARRAY[1..MAX_EOS_ORDER-2] OF EXTENDED;
     LANDAU_PARAMETERS=ARRAY[1..5] OF EXTENDED;
     ELECTRONIC_PARAMETERS=ARRAY[1..3] OF EXTENDED;
     VIBRATIONAL=ARRAY[1..NUMBER_OF_FREQUENCIES] OF EXTENDED;
     CRYSTFIELD=ARRAY[1..MAX_CRYSTFIELD_LEVELS] OF EXTENDED;
     CATION_DO_PARAMETERS=ARRAY[1..6] OF EXTENDED;
     DEFECT_PARAMETERS=ARRAY[1..4] OF EXTENDED;
     VDOSPOINTS=ARRAY[1..MAX_VDOS_POINTS] OF EXTENDED;
     STR14=STRING[14];
     STR4=STRING[4];
     STR2=STRING[2];
VAR F_INPUT,F_OUTPUT:TEXT;      
                                              (* F_INPUT: file type for input file
                                                 F_OUTPUT: file type for output file
                                               *)
    ME_VERSION,NAME_INPUTFILE,NAME_OUTPUTFILE,SUBSTANCE_NAME,NAME_GNUPLOTFILE,
    NAME_VDOSFILE:ANSISTRING;
                                              (* ME_VERSION: version of this program
                                                 NAME_INPUTFILE: Name of input file
                                                 NAME_OUTPUTFILE: Name of output file
                                                 SUBSTANCE_NAME: Name of substance
                                                 NAME_GNUPLOTFILE: Name of the GnuPlot
                                                   file for plotting VDoS
                                                 NAME_VDOSFILE: Name of experimental/abinitio
                                                   file that must be plotted
                                               *)
    DK0_ST,DG0_ST:BULKMOD_DERIVATIVES;
                                              (* DK0_ST: derivatives of bulk modulus static
                                                   lattice
                                                 DG0_ST: derivatives shear modulus static
                                                   lattice
                                               *)
    LANDAU:LANDAU_PARAMETERS;
                                              (* LANDAU[1]: Landau parameter
                                                 LANDAU[2]: Clapeyron slope of critical
                                                   phase boundary
                                                 LANDAU[3]: Critical temperature at zero
                                                   pressure
                                                 If 2nd order and tricritical case:
                                                 LANDAU[4]: 3rd law entropy correction:
                                                   0=unnecessary, 1=necessary
                                                 If 1st order case:
                                                 LANDAU[4]: order parameter at transition, Q0
                                                 LANDAU[5]: 3rd law entropy correction
                                                   0=unnecessary, 1=necessary
                                               *)
    CATION_DO:CATION_DO_PARAMETERS;
                                              (* Model of O''Neill and Navrotsky (1983,1984)
                                                 Interchange enthalpy and interchange entropy are
                                                 expressed in the sitefraction (y) of the metallic
                                                 ion as:
                                                   H_int:=(h1+h2*y)*y; S_int=(s1+s2*y)*y
                                                 CATION_DO[1]: 1st coefficient interchange enthalpy (h1)
                                                 CATION_DO[2]: 2nd coefficient interchange enthalpy (h2)
                                                 CATION_DO[3]: 1st coefficient interchange entropy  (s1)
                                                 CATION_DO[4]: 1st coefficient interchange entropy  (s2)
                                                 CATION_DO[5]: # moles of tetrahedral sites/molecular formula
                                                 CATION_DO[6]: # moles of octahedral sites/molecular formula
                                               *)
    DEFECT:DEFECT_PARAMETERS;
                                              (* Model of Dorogokupets & Oganov 2007 Phys Rev B 75:024115
                                                 Helmholtz energy contribution:
                                                 A=-(3/2)nRT*exp(s*eta^f-h*eta^g/T)
                                                 DEFECT[1]: h: enthalpy, unit Kelvin
                                                 DEFECT[2]: s: entropy,  no unit
                                                 DEFECT[3]: f: no unit
                                                 DEFECT[4]: g: no unit
                                                 eta=V/V0, n=# atoms per formula unit
                                               *)
    MOLAR_MASS,N_MOL,U_REF,V0,V0_ST,K0_ST,G0_ST,NS0,T_START,T_END,T_STEP,P_START,
    P_END,P_STEP,TEMPERATURE,PRESSURE,MAX_ARG_EXP,RESTRICT_EXP,VMEMORY,OLD_VMEMORY,
    GAMMA_EL,T_CURIE,MAG_MOMENT,NUM_MAG_ATOMS,INDEN_P,INDEN_A1,INDEN_A2,
    INDEN_M,INDEN_N,NUM_CF_ATOMS,MAGNETIC_CORR_FACTOR,S_ISENTROPE,T_HUGO_1BAR,
    P_CLONE,T_CLONE:EXTENDED;
                                              (* MOLAR_MASS: mass of the substance, gr/mol
                                                 N_MOL: Number of atoms in molecular formula
                                                   unit
                                                 U_REF: reference energy
                                                 V0: volume at zero Kelvin, zero Pascal
                                                 V0_ST: static volume at zero Pascal
                                                 K0_ST: bulk modulus of static lattice
                                                 G0_ST: shear modulus of static lattice
                                                 NS0: vibrational parameter shear modulus at
                                                   zero Kelvin, zero pressure
                                                 T_START: start temperature
                                                 T_END: end temperature
                                                 T_STEP: step in temperature
                                                 P_START: start pressure
                                                 P_END: end pressure
                                                 P_STEP: step in pressure
                                                 TEMPERATURE: temperature
                                                 PRESSURE: External pressure
                                                 MAX_ARG_EXP: maximum value of argument of
                                                   exp(x) function
                                                 RESTRICT_EXP: restrict exp(x) to 1e+1200 in
                                                   cases of anharmonicity
                                                 VMEMORY: last calculated volume
                                                 OLD_VMEMORY: for last calculated volume
                                                 GAMMA_EL: electronic Gruneisen
                                                   parameter
                                                 T_CURIE: magnetic Curie/Neeel temperature
                                                 MAG_MOMENT: magnetic moment in bohr magneton
                                                 NUM_MAG_ATOMS: magnetic: number of magnetic
                                                   atoms in the formula unit
                                                 INDEN_P: magnetic: ratio energy above critical
                                                   point to total magnetic energy
                                                 INDEN_A1: magnetic: constant in Cv expression
                                                   below tau=1
                                                 INDEN_A2: magnetic: constant in Cv expression
                                                   above tau=1
                                                 INDEN_M: magnetic: exponent of tau=T/Tc in Cv 
                                                   expression below tau=1
                                                 INDEN_N: magnetic: exponent of tau=T/Tc in Cv 
                                                   expression above tau=1
                                                 NUM_CF_ATOMS: number of atoms in a formula unit
                                                   that have energy levels split due to crystal
                                                   field
                                                 MAGNETIC_CORR_FACTOR: magnetic correction
                                                   factor, mf, such that the magnetic Helmholtz
                                                   is A=na.RT.g(t)*mf*ln(1+b)
                                                 S_ISENTROPE: Constant entropy value along
                                                   isentrope
                                                 T_HUGO_1BAR: sample temperature at 1 bar when
                                                   Hugoniot is calculated
                                                 P_CLONE: pressure condition for cloning
                                                 T_CLONE: temperaure condition for cloning
                                               *)
    STOP_CALC_KEY,INPUT_OKAY,GET_V0ST,ATTRAC_REP,EXTRAPOLATE,VDOS_PRESENT,LINUX:BOOLEAN;   
                                              (* STOP_CALC_KEY: True when calculation must
                                                   be stopped by user. False when calculation
                                                   must continue
                                                 INPUT_OKAY: True when input file is Okay
                                                 GET_V0ST: Standardly False. True when static
                                                   volume must be corrected due to change in
                                                   model parameters (by user)
                                                 ATTRAC_REP: False when searching volume by
                                                   minimization of abs(pressure). True when
                                                   normal pressure expression is used
                                                 EXTRAPOLATE: True when volume must be estimated
                                                   extrapolating from preceding calculations.
                                                   False when V0 must be taken as the start
                                                   volume
                                                 VDOS_PRESENT: experimental/abinitio VDoS
                                                   is present for plotting
                                                 LINUX: True when 'me' runs in Linux, False when
                                                  'me' runs in Windows
                                               *)
    EOS_TYPE,EOS_ORDER,LANDAU_YES,ELECTRONIC_YES,N_EINSTEIN,GRUNEISEN_TYPE,
    CALCULATION_TYPE,TABLE_SCREEN,MAGNETIC_YES,NUM_MCLAURIN_TERMS,M1_LEVELS,M2_LEVELS,
    M3_LEVELS,NUM_CF_SITES,VDOS_UNIT_DESIRED,VDOS_UNIT_EXP,NUM_VDOS_POINTS,
    CATION_DISORDER_YES,DEFECT_YES,NUMEIN_CLONE:INTEGER;
                                              (* EOS_TYPE: Equation of state type: 1=Vinet,
                                                   2=Birch-Murnaghan
                                                 EOS_ORDER: Order of the Equation of State
                                                 LANDAU_YES: 1=when Landau description is
                                                   present, 0=when it is not present
                                                 ELECTRONIC_YES: 1=electronic coefficients
                                                   0=no electronic contributions
                                                 N_EINSTEIN: Number of Einstein frequencies
                                                 GRUNEISEN_TYPE: Type of frequency/Grunesien
                                                   expression: 1=Al'tshuler, 2=Stixrude's
                                                 CALCULATION_TYPE: 1=isobar, 2=isotherm
                                                 TABLE_SCREEN: True when output is written to
                                                   screen and to file. False when output is
                                                   written to file only
                                                 MAGNETIC_YES: 1=magnetic contribution
                                                   0=no magnetic contribution
                                                 NUM_MCLAURIN_TERMS: magnetic: number of terms
                                                   in the McLaurin expression for Cv
                                                 M1_LEVELS,M2_LEVELS,M3_LEVELS: number of
                                                   energy levels in the electronic crystal
                                                   field description
                                                 NUM_CF_SITES: number of sites in the crystal
                                                   that may have different energy levels in
                                                   the crystal field description
                                                 VDOS_UNIT_DESIRED: Desired VDoS unit
                                                 VDOS_UNIT_EXP: VDoS unit of experimental/abinitio
                                                   data
                                                 NUM_VDOS_POINTS: number of points in experimental/
                                                   abinitio VDoS
                                                 CATION_DISORDER_YES: 0=no contribution,
                                                   1=cation disorder contribution
                                                 DEFECT_YES: 0=no contribution,
                                                   1=Monovacancy defects contribution
                                                 NUMEIN_CLONE: desired number of Einstein
                                                   frequencies in the VDoS: < N_EINSTEIN
                                               *)
    THETA0,FRAC,GAMMA0,Q0,GAMMA_INF,ANH_A0,ANH_Z0,THETA_TV,GAMMA_TV,
    DGAMMADV,NS_STIX:VIBRATIONAL;
                                              (* THETA0: Einstein temperature, 0 K, 0 Pa
                                                 FRAC: fraction
                                                 GAMMA0: Gruneisen parameter, 0 K, 0 Pa
                                                 Q0: mode-q(m), 0 K, 0 Pa
                                                 GAMMA_INF: Gruneisen parameter at infinite
                                                   pressure
                                                 ANH_A0: 1st anharmonicity parameter, 0K, 0 Pa
                                                 ANH_Z0: 2nd anharmonicity parameter, 0K, 0 Pa
                                                 THETA_TV: Einstein temperature at T, V
                                                 GAMMA_TV: Gruneisen parameter at T,V
                                                 DGAMMADV: (dGamma/dV)T at T,V
                                                 NS_STIX: vibrational parameter for shear
                                                   modulus at volume: it may be disperse
                                                   due to disperssion in the Gruneisen
                                                   parameters
                                               *)
   A_EL,B_EL,C_EL:ELECTRONIC_PARAMETERS;
                                              (* A_EL, B_EL, C_EL coefficients
                                                   for electronic properties,
                                                   including free electron model
                                               *)
   E_CF,G_CF,GAMMA_CF:CRYSTFIELD;
                                              (* Crystal Field E_CF, G_CF, GAMMA_CF:
                                                 E_CF: energy, cm^-1, of energy levels
                                                 G_CF: degeneracy of energy levels
                                                 GAMMA_CF: +dLn(E)/dLnV gruneisen of
                                                   energy levels
                                               *)
   X_EXP,Y_EXP:VDOSPOINTS;
                                              (* X_EXP,Y_EXP: points in experimental/
                                                 abinitio VDoS
                                               *)
   WHAT_SLASH:CHAR;                           (* WHAT_SLASH='/' when 'me' runs in Linux
                                                  and '\' when 'me' runs in Windows
                                               *)


FUNCTION INT_POW_N(X:EXTENDED;POWER:INTEGER):EXTENDED;
FUNCTION POW_N(X,N:EXTENDED):EXTENDED;
PROCEDURE CURRENT_DATE_TIME(VAR YEAR_OUT:STR4;VAR MONTH_OUT:STR14;VAR DAY_OUT:STR4;
          VAR HOUR_OUT,MINUTE_OUT,SECOND_OUT:STR2);
PROCEDURE INITIALIZE_VARIABLES;
PROCEDURE LINUX_OR_WINDOWS;
IMPLEMENTATION

FUNCTION INT_POW_N(X:EXTENDED;POWER:INTEGER):EXTENDED;
(* Calculates power of negative and positive numbers, X.
   POW_N() can only raise positive numbers to an arbitrary power (real).
   In this function POWER has to be integer, positive or negative.
 *)
VAR I,POWABS:INTEGER;RESPOW:EXTENDED;
BEGIN
  RESPOW:=1; POWABS:=ABS(POWER);
  IF POWER=0 THEN INT_POW_N:=1
  ELSE
  BEGIN
    FOR I:=1 TO POWABS DO RESPOW:=RESPOW*X;
    IF POWER<0 THEN INT_POW_N:=1/RESPOW;
    IF POWER>0 THEN INT_POW_N:=RESPOW
  END
END;

FUNCTION POW_N(X,N:EXTENDED):EXTENDED;
(* Takes the positive value X to the power N *)
VAR Z:EXTENDED;
BEGIN
  Z:=N*LN(X);
  IF Z>MAX_ARG_EXP THEN Z:=MAX_ARG_EXP
  ELSE
    IF Z<-MAX_ARG_EXP THEN Z:=-MAX_ARG_EXP;
  POW_N:=EXP(Z)
END;

PROCEDURE DAYMONTH(I_DAY,I_MONTH:INTEGER;VAR MONTH_NAME:STR14;VAR DAY:STR4);
(* Gets the month, day *)
VAR EXTENSION:STRING[2];
BEGIN
   CASE I_MONTH OF
     1:MONTH_NAME:='January';
     2:MONTH_NAME:='February';
     3:MONTH_NAME:='March';
     4:MONTH_NAME:='April';
     5:MONTH_NAME:='May';
     6:MONTH_NAME:='June';
     7:MONTH_NAME:='July';
     8:MONTH_NAME:='August';
     9:MONTH_NAME:='September';
     10:MONTH_NAME:='October';
     11:MONTH_NAME:='November';
     12:MONTH_NAME:='December'
   END;
   EXTENSION:='th';
   IF (I_DAY=1) OR (I_DAY=21) OR (I_DAY=31) THEN EXTENSION:='st';
   IF (I_DAY=2) OR (I_DAY=22) THEN EXTENSION:='nd';
   IF (I_DAY=3) OR (I_DAY=23) THEN EXTENSION:='rd';
   STR(I_DAY,DAY); DAY:=DAY+EXTENSION
end;

PROCEDURE CURRENT_DATE_TIME(VAR YEAR_OUT:STR4;VAR MONTH_OUT:STR14;VAR DAY_OUT:STR4;
          VAR HOUR_OUT,MINUTE_OUT,SECOND_OUT:STR2);
(* Gets the date and stores it in integers: year=2016; month=2=february;
   dag=0=sunday.
 *)
VAR ST:TSYSTEMTIME;
    I_YEAR,I_MONTH,I_DAY,I_HOUR,I_MINUTE,I_SECOND:INTEGER;
BEGIN
  DATETIMETOSYSTEMTIME(NOW,ST);
  WITH ST DO
  BEGIN
    I_YEAR:=YEAR; I_MONTH:=MONTH; I_DAY:=DAY;
    I_HOUR:=HOUR; I_MINUTE:=MINUTE; I_SECOND:=SECOND
  END;
  DAYMONTH(I_DAY,I_MONTH,MONTH_OUT,DAY_OUT); STR(I_YEAR,YEAR_OUT);
  STR(I_HOUR,HOUR_OUT);
  STR(I_MINUTE,MINUTE_OUT); STR(I_SECOND,SECOND_OUT)
END;

PROCEDURE INITIALIZE_VARIABLES;
(* Initialization of model parameters *)
VAR I:INTEGER;
BEGIN
  (* Version of this program *)
  ME_VERSION:=' (Version 9: July 1st, 2021)';
  SUBSTANCE_NAME:='None'; MOLAR_MASS:=0; N_MOL:=1;
  (* static parameters *)
  U_REF:=0; V0:=0; V0_ST:=0; K0_ST:=0; G0_ST:=0;
  FOR I:=1 TO MAX_EOS_ORDER-2 DO
  BEGIN
    DK0_ST[I]:=0; DG0_ST[I]:=0
  END;
  EOS_TYPE:=1; EOS_ORDER:=3;
  (* Landau parameters *)
  LANDAU_YES:=0;
  FOR I:=1 TO 5 DO LANDAU[I]:=0;
  (* Vibrational parameters *)
  N_EINSTEIN:=0; NS0:=0; GRUNEISEN_TYPE:=2; NUMEIN_CLONE:=0;
  FOR I:=1 TO NUMBER_OF_FREQUENCIES DO
  BEGIN
    THETA0[I]:=0; FRAC[I]:=0; GAMMA0[I]:=0; Q0[I]:=0; GAMMA_INF[I]:=0;
    ANH_A0[I]:=0; ANH_Z0[I]:=0;
    THETA_TV[I]:=0; GAMMA_TV[I]:=0
  END;
  (* Calculation flags *)
  STOP_CALC_KEY:=FALSE; INPUT_OKAY:=FALSE; CALCULATION_TYPE:=1;
  T_START:=10; T_END:=1000; T_STEP:=100;
  P_START:=1E+05; P_END:=1E+05; P_STEP:=0;
  P_CLONE:=1E5; T_CLONE:=298.15;
  (* Electronic parameters *)
  ELECTRONIC_YES:=0;
  (* Maximum values to prevent run time errors:
     Assign max_arg_exp to max_arg_exp_const because exp(max_arg_exp_const) gives a runtime
     error whereas exp(max_arg_exp) does not
   *)
  FOR I:=1 TO 3 DO
  BEGIN
    A_EL[I]:=0; B_EL[I]:=0; C_EL[I]:=0
  END;
  GAMMA_EL:=0;
  MAX_ARG_EXP:=MAX_ARG_EXP_CONST;
  (* Restrict the value for exp(x) to 1e+restrict_exp_x=1e+1200 because powers are taken
     of e.g. (exp(x)-1)^4 in the evaluation of anharmonicity. The maximum extended real
     is 1e+4950
   *)
  RESTRICT_EXP:=RESTRICT_EXP_X; RESTRICT_EXP:=EXP(RESTRICT_EXP*LN(10));
  GET_V0ST:=FALSE; ATTRAC_REP:=FALSE; EXTRAPOLATE:=TRUE; TABLE_SCREEN:=1;
  VMEMORY:=V0; OLD_VMEMORY:=V0;
  (* Parameters for the magnetic contribution *)
  MAGNETIC_YES:=0; NUM_MCLAURIN_TERMS:=NUMBER_OF_MCLAURIN_TERMS;
  INDEN_A1:=0; INDEN_A2:=0; INDEN_P:=0; INDEN_M:=3; INDEN_N:=5; NUM_MAG_ATOMS:=1;
  T_CURIE:=1; MAG_MOMENT:=0;
  (* Parameters for Crystal Field *)
  M1_LEVELS:=0; M2_LEVELS:=0; M3_LEVELS:=0; NUM_CF_SITES:=0; NUM_CF_ATOMS:=1;
  MAGNETIC_CORR_FACTOR:=1;
  FOR I:=1 TO MAX_CRYSTFIELD_LEVELS DO
  BEGIN
    E_CF[I]:=0; G_CF[I]:=0; GAMMA_CF[I]:=0
  END;
  (* Parameters for cation disorder model *)
  CATION_DISORDER_YES:=0;
  FOR I:=1 TO 6 DO CATION_DO[I]:=0;
  (* Parameters for defect model: monovacancy *)
  DEFECT_YES:=0;
  FOR I:=1 TO 4 DO DEFECT[I]:=0;
  (* Parameters when VDoS is treated *)
  VDOS_PRESENT:=FALSE; VDOS_UNIT_DESIRED:=1; VDOS_UNIT_EXP:=1;
  NAME_GNUPLOTFILE:=''; NAME_VDOSFILE:=''; NUM_VDOS_POINTS:=0;
  (* Initial value entropy for Isentrope calculation *)
  S_ISENTROPE:=100;
  (* Initial value of sample temperature/K when Hugoniot is calculated *)
  T_HUGO_1BAR:=300
END;

PROCEDURE LINUX_OR_WINDOWS;
(* LINUX=True when this program runs in Linux, False when it runs in windows.
   From the current directory it is seen if FitEqu run in Powershell of windows
   or in Linux
 *)
VAR DIR:ANSISTRING;
    J,LEN:INTEGER;
BEGIN
  LINUX:=TRUE; WHAT_SLASH:='/';
  DIR:=GETCURRENTDIR; LEN:=LENGTH(DIR);
  FOR J:=1 TO LEN DO
  BEGIN
    IF DIR[J]='\' THEN LINUX:=FALSE
  END;
  IF LINUX=FALSE THEN WHAT_SLASH:='\'
END;

END.
